Package: descr
Version: 1.1.9
Date: 2026-04-28
Title: Descriptive Statistics
Authors@R: c(
    person(given = "Jakson", family = "Aquino", role = "cre",
           email = "jalvesaq@gmail.com",
           comment = c(ORCID = "0000-0003-2522-8573")),
    person(given = "Dirk", family = "Enzmann", role = "aut"),
    person(given = "Marc", family = "Schwartz", role = "aut"),
    person(given = "Nitin", family = "Jain", role = "aut"),
    person(given = "Stefan", family = "Kraft", role = "aut"))
Imports: xtable, utils, grDevices, graphics, stats
Description: Weighted frequency and contingency tables of
    categorical variables and of the comparison of the mean
    value of a numerical variable by the levels of a factor,
    and methods to produce xtable objects of the tables and to
    plot them. There are also functions to facilitate
    character encoding conversion of objects, to quickly
    convert fixed-width files into CSV ones, and to export a
    data.frame to a text file with the necessary R and SPSS
    codes to reread the data.
License: GPL (>= 2)
URL: https://github.com/jalvesaq/descr
NeedsCompilation: yes
Packaged: 2026-04-28 11:33:35 UTC; aquino
Author: Jakson Aquino [cre] (ORCID: <https://orcid.org/0000-0003-2522-8573>),
  Dirk Enzmann [aut],
  Marc Schwartz [aut],
  Nitin Jain [aut],
  Stefan Kraft [aut]
Maintainer: Jakson Aquino <jalvesaq@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 12:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-28 15:30:57 UTC; unix
Archs: descr.so.dSYM

Package: descr
Version: 1.1.9
Date: 2026-04-28
Title: Descriptive Statistics
Authors@R: c(
    person(given = "Jakson", family = "Aquino", role = "cre",
           email = "jalvesaq@gmail.com",
           comment = c(ORCID = "0000-0003-2522-8573")),
    person(given = "Dirk", family = "Enzmann", role = "aut"),
    person(given = "Marc", family = "Schwartz", role = "aut"),
    person(given = "Nitin", family = "Jain", role = "aut"),
    person(given = "Stefan", family = "Kraft", role = "aut"))
Imports: xtable, utils, grDevices, graphics, stats
Description: Weighted frequency and contingency tables of
    categorical variables and of the comparison of the mean
    value of a numerical variable by the levels of a factor,
    and methods to produce xtable objects of the tables and to
    plot them. There are also functions to facilitate
    character encoding conversion of objects, to quickly
    convert fixed-width files into CSV ones, and to export a
    data.frame to a text file with the necessary R and SPSS
    codes to reread the data.
License: GPL (>= 2)
URL: https://github.com/jalvesaq/descr
NeedsCompilation: yes
Packaged: 2026-04-28 11:33:35 UTC; aquino
Author: Jakson Aquino [cre] (ORCID: <https://orcid.org/0000-0003-2522-8573>),
  Dirk Enzmann [aut],
  Marc Schwartz [aut],
  Nitin Jain [aut],
  Stefan Kraft [aut]
Maintainer: Jakson Aquino <jalvesaq@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 12:10:02 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-01 13:00:21 UTC; unix
Archs: descr.so.dSYM

useDynLib(descr, .registration = TRUE)

export(CrossTable, crosstab, LogRegR2, freq, fromUTF8, toUTF8, descr,
compmeans, histkdnc, fwf2csv, labels2R, file.head, forODFTable,
data.frame2txt)

importFrom("xtable", "xtable")
importFrom("stats", "weighted.mean", "xtabs", "chisq.test", "fisher.test",
           "mcnemar.test", "na.omit", "density", "dnorm", "sd", "pchisq",
           "quantile")
importFrom("utils", "write.table")
importFrom("grDevices", "gray.colors", "grey")
importFrom("graphics", "hist.default", "hist", "lines", "curve",
           "mosaicplot", "barplot", "boxplot", "bxp")

S3method(print, CrossTable)
S3method(print, freqtable)
S3method(print, meanscomp)
S3method(print, LogRegR2)

S3method(plot, freqtable)
S3method(plot, meanscomp)
S3method(plot, CrossTable)

S3method(dim, CrossTable)
S3method(xtable, CrossTable)
S3method(xtable, freqtable)
S3method(xtable, meanscomp)

.onLoad <- function(libname, pkgname) {
    library.dynam("descr", pkgname, libname, local = FALSE);

    if(is.null(getOption("descr.plot")))
        options(descr.plot = TRUE)
    if(is.null(getOption("descr.warn")))
        options(descr.warn = TRUE)
    if(is.null(getOption("descr.na.replacement")))
        options(descr.na.replacement = "NA")
    if(is.null(getOption("descr.round.xtabs")))
        options(descr.round.xtabs = TRUE)
}

.onUnload <- function(libpath) {
    library.dynam.unload("descr", libpath)
}


# R does not have variable labels.
descr <- function (x)
{
    if (class(x)[1] == "data.frame") {
	l <- length(x)
	bnames <- names(x)
	for (i in 1:l) {
	    lb <- attr(x[[i]], "label")
	    if (length(lb) > 0) {
		cat("\n", bnames[i], " - ", lb, "\n", sep = "")
	    }
	    else {
		cat("\n", bnames[i], "\n", sep = "")
	    }
	    print(summary(x[[i]]))
	}
	return(invisible(NULL))
    }
    else {
	lb <- attr(x, "label")
	if (length(lb) > 0) {
	    cat(deparse(substitute(x)), " - ", lb, "\n", sep = "")
	}
	print(summary(x))
	return(invisible(NULL))
    }
}


# The original versions of the functions freq, hist.kdnc, and LogRegR2 were
# written by Dirk Enzmann <dirk.enzmann@jura.uni-hamburg.de> who has given me
# permission to include them in this package. The original code can be found at
# http://www2.jura.uni-hamburg.de/instkrim/kriminologie/Mitarbeiter/Enzmann/Software/Enzmann_Software.html


# Plot histogram of variable with kernel density estimates and normal curve:
# I had to change the name because the "." was causing R to think that the
# function was a method of hist.
histkdnc <- function (v, breaks = 0, include.lowest = TRUE, right = TRUE,
    main = "Histogram with kernel density and normal curve",
    xlab = deparse(substitute(v)), col = grey(0.90),
    col.cur = c("red", "blue"), lty.cur = c(1, 1),
    xlim = NULL, ylim = NULL, ...) 
{
    v2 <- na.omit(v)
    x <- v2
    h <- hist.default(v2, plot = FALSE)
    if (length(breaks) == 1) 
	breaks <- h$breaks
    dens <- density(v2)
    argv <- list(...)
    if(is.null(ylim))
        ylim <- range(0, h$density, dnorm(x = v2, mean = mean(v2), sd = sd(v2)),
                      dens$y)
    if(is.null(xlim))
        xlim <- range(v2, dens$x)
    hist(v2, freq = FALSE, breaks = breaks, include.lowest = include.lowest, 
	right = right, xlim = xlim, ylim = ylim, col = col, 
	xlab = xlab, main = main, ...)
    lines(density(v2), col = col.cur[1], lty = lty.cur[1])
    curve(dnorm(x, mean = mean(v2), sd = sd(v2)), col = col.cur[2],
          add = TRUE, lty = lty.cur[2])
}


# Print multiple R2 analogs
print.LogRegR2 <- function(x, ...)
{
    cat(formatC(gettext("Chi2", domain = "R-descr"), flag = "-", width = 20), x$Chi2, "\n")
    cat(formatC(gettext("Df", domain = "R-descr"), flag = "-", width = 20), x$df, "\n")
    cat(formatC(gettext("Sig.", domain = "R-descr"), flag = "-", width = 20), x$p, "\n")
    cat(formatC(gettext("Cox and Snell Index", domain = "R-descr"), flag = "-", width = 20), x$CoxR2, "\n")
    cat(formatC(gettext("Nagelkerke Index", domain = "R-descr"), flag = "-", width = 20), x$NagelkerkeR2, "\n")
    cat(formatC(gettext("McFadden's R2", domain = "R-descr"), flag = "-", width = 20), x$RL2, "\n")
    return(invisible(NULL))
}

# Calculates multiple R2 analogs (pseudo R2) of logistic regression:
LogRegR2 <- function(model)
{
    if (!(model$family$family == "binomial" && (model$family$link == "logit" || model$family$link == "probit")))
	stop("No logistic regression model, no pseudo R^2 computed.")

    n    <- dim(model$model)[1]
    Chi2 <- model$null - model$dev
    Df   <- model$df.null - model$df.res
    p    <- 1-pchisq(Chi2,Df)

    Cox  <- 1-exp(-Chi2/n)             # Cox & Snell Index
    Nag  <- Cox/(1-exp(-model$null/n)) # Nagelkerke Index
    RL2  <- Chi2/model$null            # also called McFaddens R2

    x <- list('Chi2'=Chi2,'df'=Df,'p'=p,'RL2'=RL2,'CoxR2'=Cox,'NagelkerkeR2'=Nag)
    class(x) <- "LogRegR2"
    x
}

no.drop.levels <- function(x)
{
    if(sum(is.na(x)) > 0){
        nl <- length(levels(x)) + 1
        lv <- c(levels(x), options("descr.na.replacement"))
        io <- is.ordered(x)
        x <- as.numeric(x)
        x[is.na(x)] <- nl
        x <- factor(x, levels = 1:nl, labels = lv, ordered = io)
    }
    x
}

fromto <- function (x, from, to)
{
    if (is.list(x)) {
	xattr <- attributes(x)
	x <- lapply(x, fromto, from, to)
	attributes(x) <- xattr
    } else {
	if (is.factor(x)) {
	    levels(x) <- iconv(levels(x), from, to, sub = "byte")
	} else {
	    if (is.character(x))
		x <- iconv(x, from, to, sub = "byte")
	}
	lb <- attr(x, "label")
	if (length(lb) > 0) {
	    attr(x, "label") <- iconv(attr(x, "label"), from, to, sub = "byte")
	}
    }
    x
}

# Converts a variable from UTF-8 into other encoding
fromUTF8 <- function (x, to = "WINDOWS-1252")
{
    fromto(x, "UTF-8", to)
}

# Converts a variable from any encoding into UTF-8
toUTF8 <- function (x, from = "WINDOWS-1252")
{
    fromto(x, from, "UTF-8")
}

# ---------------------------------------
# Author: Stefan Kraft
#         Vienna University of Technology
# ---------------------------------------

# Code copied from "simPopulation" package version 0.4.0

quantileWt <- function(x, weights = NULL, 
        probs = seq(0, 1, 0.25), na.rm = TRUE) {
    # initializations
    if(!is.numeric(x)) stop("'x' must be a numeric vector")
    x <- unname(x)  # unlike 'quantile', this never returns a named vector
    if(is.null(weights)) {
        return(quantile(x, probs, na.rm=na.rm, names=FALSE, type=1))
    } else if(!is.numeric(weights)) stop("'weights' must be a numeric vector")
    else if(length(weights) != length(x)) {
        stop("'weights' must have the same length as 'x'")
    } else if(!all(is.finite(weights))) stop("missing or infinite weights")
    if(!is.numeric(probs) || all(is.na(probs)) || 
            isTRUE(any(probs < 0 | probs > 1))) {
        stop("'probs' must be a numeric vector with values in [0,1]")
    }
    if(length(x) == 0) return(rep.int(NA, length(probs)))
    if(!isTRUE(na.rm) && any(is.na(x))) {
        stop("missing values and NaN's not allowed if 'na.rm' is not TRUE")
    }
    # sort values and weights
    ord <- order(x, na.last=NA)
    x <- x[ord]
    weights <- weights[ord]
    # some preparations
    rw <- cumsum(weights)/sum(weights)
    # obtain quantiles
    select <- sapply(probs, function(p) min(which(rw >= p)))
    q <- x[select]
    return(q)
}

spBwplotStats <- function(x, weights = NULL, coef = 1.5, 
        zeros = TRUE, do.out = TRUE) {
    # initializations
    if(!is.numeric(x)) stop("'x' must be a numeric vector")
    if(!is.numeric(coef) || length(coef) != 1 || coef < 0) {
        stop("'coef' must be a single non-negative number")
    }
    # get quantiles
    if(isTRUE(zeros)) {
        zero <- ifelse(is.na(x), FALSE, x == 0)
        x <- x[!zero]
        if(is.null(weights)) nzero <- sum(zero)
        else {
            # if 'zeros' is not TRUE, these checks are done in 'quantileWt'
            # but here we need them since we use subscripting
            if(!is.numeric(weights)) stop("'weights' must be a numeric vector")
            else if(length(weights) != length(zero)) {
                stop("'weights' must have the same length as 'x'")
            }
            nzero <- sum(weights[zero])
            weights <- weights[!zero]
        }
    } else nzero <- NULL
    ok <- !is.na(x)
    n <- if(is.null(weights)) sum(ok) else sum(weights[ok])
    if(n == 0) stats <- rep.int(NA, 5)
    else stats <- quantileWt(x, weights)
    iqr <- diff(stats[c(2, 4)])  # inter quartile range
    if(coef == 0) do.out <- FALSE
    else {
        if(is.na(iqr)) out <- is.infinite(x) 
        else {
            lower <- stats[2] - coef * iqr
            upper <- stats[4] + coef * iqr
            out <- ifelse(ok, x < lower | x > upper, FALSE)
        }
        if(any(out)) stats[c(1, 5)] <- range(x[!out], na.rm=TRUE)
    }
    res <- list(stats=stats, n=n, nzero=nzero, 
        out=if(isTRUE(do.out)) x[out] else numeric())
    class(res) <- "spBwplotStats"
    res
}

freq <- function (x, w, user.missing, plot = getOption("descr.plot"), ...)
{
    xlab <- attr(x, "label", TRUE)
    if(is.null(xlab))
        xlab <- deparse(substitute(x))
    if (is.factor(x) == FALSE)
        x <- as.factor(x)
    xclass <- class(x)

    if (missing(w))
        w <- rep(1, length(x))

    nmiss <- sum(is.na(x))
    xlevels <- levels(x)
    l <- length(xlevels)
    hasna <- FALSE
    xv <- x
    if (nmiss) {
        hasna <- TRUE
        l <- l + 1
        xlevels[l] <- "NA's"
        x <- as.numeric(x)
        x[is.na(x)] <- l
        x <- factor(x, levels=1:l, labels = xlevels)
    }

    xfreq <- tapply(w, x, sum, na.rm = TRUE)
    xfreq[is.na(xfreq)] <- 0
    xtotal <- sum(xfreq, na.rm = TRUE)
    xperc <- 100 * xfreq / xtotal

    ftab <- cbind(xfreq, xperc)
    cnames <- c(gettext("Frequency", domain = "R-descr"),
                gettext("Percent", domain = "R-descr"))

    xvfreq <- xfreq
    if(nmiss){
        xvfreq[xlevels == "NA's"] <- NA
    }
    if(!missing(user.missing)){
        user.missing <- paste("^", user.missing, "$", sep = "")
        for(lev in user.missing){
            idx <- grep(lev, xlevels)
            if(length(idx))
                xvfreq[idx] <- NA
        }
    }

    if(nmiss || !missing(user.missing)){
        xvtotal <- sum(xvfreq, na.rm = TRUE)
        xvperc <- 100 * xvfreq / xvtotal
        ftab <- cbind(ftab, xvperc)
        cnames <- c(cnames, gettext("Valid Percent", domain = "R-descr"))
    }

    if(xclass[1] == "ordered"){
        if(nmiss || !missing(user.missing)){
            xxvperc <- xvperc
            xxvperc[is.na(xxvperc)] <- 0
            xvcumsum <- cumsum(xxvperc)
            xvcumsum[is.na(xvperc)] <- NA
        } else
            xvcumsum <- cumsum(xperc)
        ftab <- cbind(ftab, xvcumsum)
        cnames <- c(cnames, gettext("Cum Percent", domain = "R-descr"))
    }

    total <- apply(ftab, 2, sum, na.rm = TRUE)
    if(xclass[1] == "ordered")
        total["xvcumsum"] <- NA
    ftab <- rbind(ftab, total)

    rnames <- levels(x)
    rnames[l + 1] <- gettext("Total", domain = "R-descr")

    colnames(ftab) <- cnames
    rownames(ftab) <- rnames

    attr(ftab, "xlab") <- xlab
    class(ftab) <- c("freqtable", "matrix")

    # Attributes for plotting
    if(nmiss || !missing(user.missing))
        xdata.c <- xvfreq
    else
        xdata.c <- xfreq
    if(length(grep("^NA's$", names(xdata.c))) > 0)
        xdata.c["NA's"] <- NA
    xdata.c <- xdata.c[!is.na(xdata.c)]
    if(nmiss || !missing(user.missing))
        xdata.p <- xvperc
    else
        xdata.p <- xperc
    if(length(grep("^NA's$", names(xdata.p))) > 0)
        xdata.p["NA's"] <- NA
    xdata.p <- xdata.p[!is.na(xdata.p)]

    attr(ftab, "xdata.c") <- xdata.c
    attr(ftab, "xdata.p") <- xdata.p

    if(plot == TRUE)
        plot.freqtable(ftab, ...)

    ftab
}

print.freqtable <- function(x, digits = 4, na.print="", ...){
    xlab <- attr(x, "xlab")
    cat(xlab, "\n")
    attr(x, "xlab") <- NULL
    attr(x, "xdata.c") <- NULL
    attr(x, "xdata.p") <- NULL
    class(x) <- "matrix"
    print(x, digits = digits, na.print = na.print, ...)
    return(invisible(NULL))
}

plot.freqtable <- function(x, y.axis = "count", ...)
{
    if(y.axis == "count"){
        xdata <- attr(x, "xdata.c")
    } else if(y.axis == "percent"){
        xdata <- attr(x, "xdata.p")
    } else {
        msg <- paste(gettext("Invalid y.axis: '", domain = "R-descr"),
                     y.axis[1], "'", sep = "")
        stop(msg)
    }
    barplot(xdata, ...)
}

forODFTable <- function(x, digits = 1, ...)
{
    if (!isa(x, "CrossTable")) {
        msg <- sprintf(gettext("'%s' should be of class 'CrossTable'.",
                               domain = "R-descr"), deparse(substitute(x)))
        stop(msg)
    }

    if(x$format == "SPSS")
        hdd <- 100
    else
        hdd <- 1
    nr <- dim(x$tab)[1]
    nc <- dim(x$tab)[2]

    tab <- format(x$tab, ...)
    if(x$expected == TRUE){
        xex <- outer(x$rs, x$cs, "*")
        xex <- xex / x$gt
        xx <- format(round(xex, digits), trim = TRUE, ...)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(x$prop.chisq){
        xx <- ((x$CST$expected - x$tab) ^ 2) / x$CST$expected
        xx <- format(round(xx, digits), trim = TRUE, ...)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(!is.na(x$prop.row[1])){
        xx <- format(round(x$prop.row * hdd, digits), trim = TRUE, ...)
        if(hdd == 100)
            xx <- matrix(paste(xx, "%", sep = ""), nrow = nr, ncol = nc)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(!is.na(x$prop.col[1])){
        xx <- format(round(x$prop.col * hdd, digits), trim = TRUE, ...)
        if(hdd == 100)
            xx <- matrix(paste(xx, "%", sep = ""), nrow = nr, ncol = nc)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(!is.na(x$prop.tbl[1])){
        xx <- format(round(x$prop.tbl * hdd, digits), trim = TRUE, ...)
        if(hdd == 100)
            xx <- matrix(paste(xx, "%", sep = ""), nrow = nr, ncol = nc)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(!is.na(x$resid) && x$resid == TRUE && x$expected == TRUE){
        xx <- x$tab - xex
        xx <- format(round(xx, digits), trim = TRUE, ...)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(!is.na(x$sresid) && x$sresid == TRUE && x$expected == TRUE){
        xx <- x$CST$residual
        xx <- format(round(xx, digits), trim = TRUE, ...)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    if(!is.na(x$asr[1])){
        xx <- format(round(x$asr, digits), trim = TRUE, ...)
        tab <- paste(tab, xx, sep = "<text:line-break/>")
        tab <- matrix(tab, nrow = length(x$rs), ncol = length(x$cs))
    }
    tab <- cbind(tab, x$rs)
    tab <- rbind(tab, c(x$cs, x$gt))
    rownames(tab)[dim(tab)[1]] <- gettext("Total", domain = "R-descr")
    colnames(tab)[dim(tab)[2]] <- gettext("Total", domain = "R-descr")

    tab
}

xtable.CrossTable <- function(x, caption = NULL, label = NULL,
                              align = NULL, digits = NULL,
                              display = NULL, auto = FALSE,
                              multirow = FALSE, hline = FALSE, ...)
{
    argl <- list(...)
    for(n in names(argl))
        if(n %in% names(x))
            x[[n]] <- argl[[n]]

    nr <- nrow(x$tab)
    nc <- ncol(x$t)

    nt <- CreateNewTab(x, ...)
    # Scape the % symbol if the user probably will not sanitize the text
    if(multirow || hline){
        if(x$percent)
            for(i in 1:nrow(nt))
                for(j in 1:ncol(nt))
                    nt[i, j] <- sub("%", "\\\\%", nt[i, j])
        if(x$row.labels)
            rownames(nt) <- sub("%",  "\\\\%", rownames(nt))
    }

    # Add rownames as first column
    nt <- cbind(rownames(nt), nt)
    colnames(nt)[1] <- x$RowData
    rownames(nt) <- NULL

    if(x$total.c && !is.na(x$prop.col)[1])
        nrnt <- nrow(nt) - 2
    else if(x$total.c)
        nrnt <- nrow(nt) - 1
    else
        nrnt <- nrow(nt)

    n <- nrnt / nr
    if(multirow && !x$row.labels){
        idxm <- seq(1, nrnt, n)
        nt[idxm, 1] <- paste("\\multirow{", n, "}{*}{", nt[idxm, 1], "}", sep = "")
    }
    if(hline){
        idxh <- seq(n+1, nrnt+1, n)
        idxh <- idxh[idxh < nrow(nt)] # necessary when total.c = FALSE
        nt[idxh, 1] <- paste("\\hline\n", nt[idxh, 1], sep = "")
    }

    if(multirow){
        idxc <- 1:ncol(x$tab) + 1
        colnames(nt)[idxc] <- paste0("\\multicolumn{1}{c}{", colnames(nt)[idxc], "}")
        col1txt <- paste0("\\multirow{2}{*}{", gsub("\\$", "\\\\$", x$RowData), "} & \\multicolumn{", ncol(x$tab), "}{c}{", gsub("\\$", "\\\\$", x$ColData), "}")
        if(x$total.r)
            col1txt <- paste0(col1txt, " & \\multirow{2}{*}{", colnames(nt)[ncol(nt)], "}\\\\\n \\cline{2-", ncol(x$tab)+1,"}", sep = "")
        else
            col1txt <- paste0(col1txt, " \\\\\n \\cline{2-", ncol(x$tab)+1,"}")
        colnames(nt)[1] <- col1txt
        if(x$total.r)
            colnames(nt)[ncol(nt)] <- " "
    }

    if(is.null(align))
        align = paste0("ll", paste(rep("r", ncol(nt) - 1), collapse = ""))
    xtable::xtable(nt, caption = caption, label = label, align = align,
                   digits = digits, display = display, auto = auto, ...)
}

xtable.freqtable <- function(x, caption = NULL, label = NULL, align = NULL,
                             digits = 1, display = NULL, ...)
{
    if(is.null(align))
        align <- paste0("l", paste0(rep("r", ncol(x)), collapse = ""))
    if(is.null(display))
        display <- c("s", "d", rep("f", ncol(x) - 1))
    xtable::xtable(unclass(x), caption=caption, label=label, align=align,
                   display=display, ...)
}

xtable.meanscomp <- function(x, caption = NULL, label = NULL, align = NULL,
                             digits = 1, display = NULL, ...)
{
    if(is.null(align))
        align <- "lrrr"
    if(is.null(display))
        display <- c("s", "f", "d", "f")
    xtable::xtable(unclass(x), caption=caption, label=label, align=align,
                   display=display, ...)
}

# These functions were developed from the function CrossTable of the package
# gmodels.  The original function had the following comments:
#
# Revision 2.2 2006/05/02
# Fix a bug when a matrix is passed as the 'x' argument
# Reported by Prof. Albert Sorribas same day
# Fix involved creating default values for RowData and ColData
# when there are no dimnames for the matrix

# Revision 2.1 2005/06/26
# Added 'dnn' argument to enable specification of dimnames
# as per table()
# Correct bug in SPSS output for 1d table, where proportions
# were being printed and not percentages ('%' output)

# Revision 2.0 2005/04/27
# Added 'format = "d"' to all table count output
# so that large integers do not print in
# scientific notation

GetDigitsList <- function(x)
{
    if(is.list(x$digits))
        dgts <- list(expected = ifelse(is.null(x$digits$expected), 1, x$digits$expected),
                     prop = ifelse(is.null(x$digits$prop), 1, x$digits$prop),
                     percent = ifelse(is.null(x$digits$percent), 1, x$digits$percent),
                     others = ifelse(is.null(x$digits$others), 1, x$digits$others))
    else
        dgts <- list(expected = x$digits[1], prop = x$digits[1],
                     percent = x$digits[1],  others = x$digits[1])
    dgts
}

CreateNewTab <- function(x, ...)
{
    nr <- dim(x$tab)[1]
    nc <- dim(x$tab)[2]
    nt <- cbind(x$tab, x$rs)
    colnames(nt)[ncol(nt)] <- gettext("Total", domain = "R-descr")
    nt <- format(nt, scientific = FALSE)

    dgts <- GetDigitsList(x)

    if(x$format == "SPSS"){
        hdd <- 100
        dgts$prop <- dgts$percent
    } else {
        hdd <- 1
        x$percent <- FALSE
    }

    appendlines <- function(nt, xx, rowlab, prct = FALSE, hasttl = FALSE)
    {
        if(prct)
            for(i in 1:nrow(xx))
                for(j in 1:ncol(xx))
                    xx[i, j] <- paste0(xx[i, j], "%")
        if(!hasttl)
            xx <- cbind(xx, rep("", nr))
        if(!x$row.labels)
            rowlab <- " "
        rownames(xx) <- rep(rowlab, nrow(xx))
        n <- dim(nt)[1] / nr
        nt <- rbind(nt, xx)
        idx <- integer()
        k <- 1
        l <- nr * n + 1
        for(i in 1:nr){
            for(j in 1:n){
                idx <- c(idx, k)
                k <- k + 1
            }
            idx <- c(idx, l)
            l <- l + 1
        }
        nt <- nt[idx, ]
        nt
    }

    if(x$expected){
        xx <- x$CST$expected
        xx <- format(round(xx, dgts$expected), scientific = FALSE, trim = TRUE, ...)
        nt <- appendlines(nt, xx, gettext("expected", domain = "R-descr"))
    }

    if(x$prop.chisq){
        xx <- ((x$CST$expected - x$tab) ^ 2) / x$CST$expected
        xx <- format(round(xx, digits = dgts$others), trim = TRUE, ...)
        nt <- appendlines(nt, xx, gettext("chisq", domain = "R-descr"))
    }

    if(!is.na(x$prop.row[1])){
        xx <- cbind(x$prop.row, x$rs / x$gt)
        xx <- format(round(xx * hdd, digits = dgts$prop), trim = TRUE, ...)
        if(hdd == 100)
            nt <- appendlines(nt, xx, gettext("row %", domain = "R-descr"),
                              x$percent, TRUE)
        else
            nt <- appendlines(nt, xx, gettext("row prop.", domain = "R-descr"),
                              x$percent, TRUE)
    }

    if(!is.na(x$prop.col[1])){
        xx <- format(round(x$prop.col * hdd, digits = dgts$prop), trim = TRUE, ...)
        if(hdd == 100)
            nt <- appendlines(nt, xx, gettext("col %", domain = "R-descr"),
                              x$percent)
        else
            nt <- appendlines(nt, xx, gettext("col prop.", domain = "R-descr"),
                              x$percent)
    }

    if(!is.na(x$prop.tbl[1])){
        xx <- format(round(x$prop.tbl * hdd, digits = dgts$prop), trim = TRUE, ...)
        if(hdd == 100)
            nt <- appendlines(nt, xx, gettext("table %", domain = "R-descr"),
                              x$percent)
        else
            nt <- appendlines(nt, xx, gettext("table prop.", domain = "R-descr"),
                              x$percent)
    }

    if(!is.na(x$resid) && x$resid == TRUE){
        xx <- x$tab - x$CST$expected
        xx <- format(round(xx, digits = dgts$others), scientific = FALSE, trim = TRUE, ...)
        nt <- appendlines(nt, xx, gettext("residual", domain = "R-descr"))
    }

    if(!is.na(x$sresid) && x$sresid == TRUE){
        xx <- x$CST$residual
        xx <- format(round(xx, digits = dgts$others), trim = TRUE, ...)
        nt <- appendlines(nt, xx, gettext("std. res.", domain = "R-descr"))
    }

    if(!is.na(x$asr[1])){
        xx <- format(round(x$asr, digits = dgts$others), trim = TRUE, ...)
        nt <- appendlines(nt, xx, gettext("adj. std. res.", domain = "R-descr"))
    }

    if(x$total.c){
        nt <- rbind(nt, format(c(x$cs, x$gt), scientific = FALSE))
        rownames(nt)[nrow(nt)] <- gettext("Total", domain = "R-descr")

        # Add final row if necessary
        if(!is.na(x$prop.col[1])){
            xx <- format(round(hdd * x$cs / x$gt, digits = dgts$prop), trim = TRUE, ...)
            if(hdd == 100 && x$percent)
                xx <- paste0(xx, "%")
            nt <- rbind(nt, c(xx, ""))
        }
    }

    # Delete last column if necessary
    if(!x$total.r)
        nt <- nt[, 1:(ncol(nt)-1)]

    tdim <- dim(nt)
    tdnn <- list(rownames(nt), colnames(nt))
    names(tdnn) <- c(attr(nt, "RowData"), attr(nt, "ColData"))
    attributes(nt) <- NULL
    attr(nt, "dim") <- tdim
    dimnames(nt) <- tdnn
    nt
}

Abbrev1 <- function(x, lmt)
{
    if(nchar(x) <= lmt)
        return(x)
    xc <- charToRaw(x)
    len1 <- length(xc)

    # Two consecutive spaces
    i <- 1
    while(i < len1){
        if(xc[i] == 0x20 && xc[i+1] == 0x20)
            return(rawToChar(xc[-i]))
        i <- i + 1
    }

    # Trailing space or tab
    if(xc[len1] == 0x20 || xc[len1] == 0x09)
        return(rawToChar(xc[-len1]))

    # Last lower case ascii vowel (unless it is the first letter in a word)
    i <- len1
    while(i > 1){
        if(xc[i-1] != 0x20 && (xc[i] == 0x61 || xc[i] == 0x65 || xc[i] == 0x69 || xc[i] == 0x6f || xc[i] == 0x75)){
            xc <- xc[-i]
            return(rawToChar(xc))
        }
        i <- i - 1
    }

    # Last lower case ascii letter (unless it is the first letter in a word)
    i <- len1
    while(i > 0){
        if(xc[i-1] != 0x20 && xc[i] > 0x60 && xc[i] < 0x7b){
            xc <- xc[-i]
            return(rawToChar(xc))
        }
        i <- i - 1
    }

    # Last letter
    x <- unlist(strsplit(x, ""))
    len <- length(x)
    x <- x[-len]
    paste0(x, collapse = "")
}

CrossTable <- function (x, y,
    digits = list(expected = 1, prop = 3, percent = 1, others = 3),
    max.width = NA, expected = FALSE,
    prop.r = TRUE, prop.c = TRUE, prop.t = TRUE, prop.chisq = TRUE,
    chisq = FALSE, fisher = FALSE, mcnemar = FALSE, resid = FALSE,
    sresid = FALSE, asresid = FALSE, missing.include = FALSE,
    drop.levels = TRUE, format = c("SAS", "SPSS"), dnn = NULL,
    cell.layout = TRUE, row.labels = !cell.layout,
    percent = (format == "SPSS" && !row.labels),
    total.r, total.c, xlab = NULL, ylab = NULL, ...)
{
    format = match.arg(format)

    RowData <- deparse(substitute(x))
    if (missing(y))
        ColData <- NA
    else
        ColData <- deparse(substitute(y))

    ## Ensure that max.width >= 1
    if (!is.na(max.width) && max.width < 1)
        stop("max.width must be >= 1")
    ## Set 'x' vector flag
    vector.x <- FALSE

    if (missing(y))
    {
        ## is x a vector?
        if (is.null(dim(x)))
        {
            if (missing.include)
                x <- no.drop.levels(x)
            if (drop.levels)
                x <- factor(x)
            tab <- t(as.matrix(table(x)))
            vector.x <- TRUE
        }
        ## is x a matrix?
        else if (length(dim(x) == 2))
        {
            if(any(x < 0) || any(is.na(x)))
                stop("all entries of x must be nonnegative and finite")

            ## Check to see if x has names(dimnames) defined. If yes, use these for
            ## 'RowData' and 'ColData' labels, else create blank ones
            ## This can be overridden by setting 'dnn' values
            if (is.null(names(dimnames(x))))
            {
                RowData <- ""
                ColData <- ""
            } else {
                RowData <- names(dimnames(x))[1]
                ColData <- names(dimnames(x))[2]
            }

            ## Add generic column and rownames if required
            ## check each separately, in case user has defined one or the other
            if (is.null(rownames(x)))
                rownames(x) <- paste("[", 1:nrow(x), ",]", sep = "")
            if (is.null(colnames(x)))
                colnames(x) <- paste("[,", 1:ncol(x), "]", sep = "")

            tab <- x
        }
        else
            stop("x must be either a vector or a 2 dimensional matrix, if y is not given")
    } else {
        if(missing.include){
            x <- no.drop.levels(x)
            y <- no.drop.levels(y)
        } 
        if(drop.levels){
            x <- factor(x)
            y <- factor(y)
        }

        if(length(x) != length(y))
            stop("x and y must have the same length")

        ## Generate table
        tab <- table(x, y)
    }

    ## Check dnn, and use it
    if(!is.null(dnn)){
        if(all(dim(tab) >= 2) && length(dnn) != 2)
            stop("dnn must have length of 2, one element for each table dimension")
        RowData <- dnn[1]
        if(length(dnn) > 1)
            ColData <- dnn[2]
    }

    ## if tab is not at least a 2 x 2, do not do stats
    ## even if any set to TRUE. Do not do col/table props
    if (any(dim(tab) < 2))
        prop.c <- prop.chisq <- chisq <- expected <- fisher <- mcnemar <- FALSE

    if (vector.x && dim(tab)[2] < 2)
        prop.r <- FALSE
    if (!vector.x && dim(tab)[1] < 2)
        prop.r <- FALSE

    CPR <- CPC <- CPT <- GT <- RS <- TotalN <- CSTc <- CST <- ASR <- FTt <-
        FTl <- FTg <- McN <- McNc <- NA

    ## Generate cell proportion of row
    if(prop.r)
        CPR <- prop.table(tab, 1)

    ## Generate cell proportion of col
    if(prop.c)
        CPC <- prop.table(tab, 2)

    ## Generate cell proportion of total
    if(prop.t)
        CPT <- prop.table(tab)

    ## Generate summary counts
    GT <- sum(tab)
    RS <- rowSums(tab)
    CS <- colSums(tab)

    if (length(dim(x) == 2))
        TotalN <- GT
    else
        TotalN <- length(x)

    ## Perform Chi-Square Tests
    if (expected || chisq || prop.chisq || resid || sresid || asresid) {
        if(!chisq && !prop.chisq){
            wv <- getOption("warn")
            options(warn = -1)
        }
        CST <- chisq.test(tab, correct = FALSE, ...)
        if (all(dim(tab) == 2))
            CSTc <- chisq.test(tab, correct = TRUE, ...)
        if(!chisq && !prop.chisq){
            options(warn = wv)
        }
    }

    if (asresid & !vector.x)
        ASR <- (CST$observed-CST$expected)/sqrt(CST$expected*((1-RS/GT) %*% t(1-CS/GT)))

    if (fisher)
    {
        try(FTt <- fisher.test(tab, alternative = "two.sided"))
        if (all(dim(tab) == 2))
        {
            FTl <- fisher.test(tab, alternative = "less")
            FTg <- fisher.test(tab, alternative = "greater")
        }
    }

    if (mcnemar)
    {
        if(dim(tab)[1] == dim(tab)[2])
            McN <- mcnemar.test(tab, correct = FALSE)
        if (all(dim(tab) == 2))
            McNc <- mcnemar.test(tab, correct = TRUE)
    }

    if(!missing(total.r)){
        if(!is.logical(total.r))
            stop(gettext("total.r must be logical", domain = "R-descr"))
        if(missing(total.c))
            total.c <- total.r
    }
    if(!missing(total.c)){
        if(!is.logical(total.c))
            stop(gettext("total.c must be logical", domain = "R-descr"))
        if(missing(total.r))
            total.r <- total.c
    }
    if(missing(total.r) & missing(total.c))
        total.r <- total.c <- TRUE

    res <- list(tab = tab, prop.row = CPR, prop.col = CPC, prop.tbl = CPT,
                gt = GT, rs = RS, cs = CS, total.n = TotalN, chisq = chisq,
                CST = CST, chisq.corr = CSTc, fisher.ts = FTt,
                fisher.lt = FTl, fisher.gt = FTg, print.mcnemar = mcnemar,
                mcnemar = McN, mcnemar.corr = McNc, asr = ASR,
                RowData = RowData, ColData = ColData, digits = digits,
                max.width = max.width, vector.x = vector.x,
                expected = expected, prop.chisq = prop.chisq, resid = resid,
                sresid = sresid, asresid = asresid, format = format,
                cell.layout = cell.layout, row.labels = row.labels,
                percent = percent, total.r = total.r, total.c = total.c)

    # Add "t" element to avoid breaking pander package which manipulates the
    # CrossTable object:
    res$t <- res$tab

    # Attributes for plotting
    attr(res, "xlab") <- xlab
    attr(res, "ylab") <- ylab

    class(res) <- "CrossTable"
    res
}

print.CrossTable <- function(x, ...)
{
    argl <- list(...)
    for(n in names(argl))
        if(n %in% names(x))
            x[[n]] <- argl[[n]]

    nt <- CreateNewTab(x, ...)
    tab <- x$tab
    CPR <- x$prop.row
    CPC <- x$prop.col
    CPT <- x$prop.tbl
    GT <- x$gt
    RS <- x$rs
    CS <- x$cs
    TotalN <- x$total.n
    chisq <- x$chisq
    CST <- x$CST
    CSTc <- x$chisq.corr
    FTt <- x$fisher.ts
    FTl <- x$fisher.lt
    FTg <- x$fisher.gt
    McN <- x$mcnemar
    McNc <- x$mcnemar.corr
    ASR <- x$asr
    RowData <- x$RowData
    ColData <- x$ColData
    max.width <- x$max.width
    vector.x <- x$vector.x
    expected <- x$expected
    prop.r <- !is.na(CPR[1])
    prop.c <- !is.na(CPC[1])
    prop.t <- !is.na(CPT[1])
    prop.chisq <- x$prop.chisq
    fisher <- (class(FTt) == "htest")
    resid <- x$resid
    sresid <- x$sresid
    asresid <- x$asresid
    mcnemar <- x$print.mcnemar
    format <- x$format
    cell.layout <- x$cell.layout
    total.r <- x$total.r
    total.c <- x$total.c
    outDec <- getOption("OutDec")

    dgts <- GetDigitsList(x)

    if(format == "SAS") {
        hdd <- 1
    } else {
        if (format == "SPSS") {
            hdd <- 100
            dgts$prop <- dgts$percent
        } else {
            stop("unknown format")
        }
    }

    if(vector.x)
        expected <- prop.chisq <- prop.c <- prop.t <- resid <- sresid <- asresid <- FALSE

    #### Printing the tables

    ## Print Cell Layout
    if(cell.layout){
        cat("  ", gettext("Cell Contents", domain = "R-descr"), "\n")
        cat("|-------------------------|\n")
        if (format=="SAS") {
            cat(gettext("|                       N |", domain = "R-descr"), "\n")
            if (expected)
                cat(gettext("|              Expected N |", domain = "R-descr"), "\n")
            if (prop.chisq)                                                     
                cat(gettext("| Chi-square contribution |", domain = "R-descr"), "\n")
            if (prop.r)                                                         
                cat(gettext("|           N / Row Total |", domain = "R-descr"), "\n")
            if (prop.c)                                                         
                cat(gettext("|           N / Col Total |", domain = "R-descr"), "\n")
            if (prop.t)                                                         
                cat(gettext("|         N / Table Total |", domain = "R-descr"), "\n")
        } else if (format == "SPSS") {
            cat(gettext("|                   Count |", domain = "R-descr"), "\n")
            if (expected)
                cat(gettext("|         Expected Values |", domain = "R-descr"), "\n")
            if (prop.chisq)                                                       
                cat(gettext("| Chi-square contribution |", domain = "R-descr"), "\n")
            if (prop.r)                                                           
                cat(gettext("|             Row Percent |", domain = "R-descr"), "\n")
            if (prop.c)                                                           
                cat(gettext("|          Column Percent |", domain = "R-descr"), "\n")
            if (prop.t)                                                           
                cat(gettext("|           Total Percent |", domain = "R-descr"), "\n")
        }
        if (resid)                                                            
            cat(gettext("|                Residual |", domain = "R-descr"), "\n")
        if (sresid)                                                           
            cat(gettext("|            Std Residual |", domain = "R-descr"), "\n")
        if (asresid)                                                          
            cat(gettext("|           Adj Std Resid |", domain = "R-descr"), "\n")
        cat("|-------------------------|\n")
    }

    ## Print 1 X N vector
    if (vector.x) {
        ## Set consistent column widths based upon dimnames and table values
        strt <- formatC(unclass(tab), digits = dgts$prop, format = "f", width = 0, decimal.mark = outDec)
        CWidth <- max(dgts$prop + 2, c(nchar(strt, type = "width"),
                                    nchar(dimnames(tab)[[2]], type = "width")))
        if(prop.r){
            if(vector.x)
                strt <- formatC(unclass(CPT), digits = dgts$prop, format = "f", width = 0, decimal.mark = outDec)
            CWidth <- max(CWidth, nchar(strt, type = "width"))
        }

        ## Create row separators
        RowSep <- paste(rep("-", CWidth + 2), collapse = "")

        if(is.na(max.width))
            max.width = floor((getOption("width") - 2) / (CWidth + 3))
        if (length(tab) > max.width) {
            ## set breakpoints for output based upon max.width
            final.row <- length(tab) %% max.width
            max <- length(tab) - final.row
            ## Define breakpoint indices for each row
            start <- seq(1, max, max.width)
            end <- start + (max.width - 1)
            ## Add final.row if required
            if (final.row > 0) {
                start <- c(start, end[length(end)] + 1)
                end <- c(end, end[length(end)] + final.row)
            }
        } else {
            ## Each value printed horizontally in a single row
            start <- 1
            end <- length(tab)
        }

        cat("\n")

        for (i in 1:length(start)) {
            cat("| ")
            cat(paste(formatC(dimnames(tab)[[2]][start[i]:end[i]], width = CWidth, format = "s"), collapse = " | "))
            cat(" |\n|")
            cat(rep(RowSep, (end[i] - start[i]) + 1), sep = "|")
            cat("|\n| ")
            cat(formatC(tab[, start[i]:end[i]], width = CWidth, format = "d"), sep = " | ")
            if(prop.r){
                cat(" |\n| ")
                cat(formatC(CPT[, start[i]:end[i]] * hdd, width = CWidth,
                            digits = dgts$prop, format = "f", decimal.mark = outDec), sep = " | ")
            }
            cat(" |\n|")
            cat(rep(RowSep, (end[i] - start[i]) + 1), sep = "|")
            cat("|\n\n")

        }  ## End of for (i in 1:length(start))

        if(format == "SPSS" && GT < TotalN)
            cat("\n", gettext("Number of Missing Observations:", domain = "R-descr"),
                " ", TotalN-GT, " (", 100*(TotalN-GT)/TotalN, "%)\n", sep = "")
        return(invisible(x))
    } ## End of if (vector.x)


    nr <- nrow(nt)
    nc <- ncol(nt)
    rnames <- rownames(nt)
    cnames <- colnames(nt)

    # Check column widths and fix them if necessary
    availablewidth <- getOption("width")
    minColWd <- max(nchar(nt, type = "width"))

    clabwidth <- nchar(cnames, type = "width")
    rlabwidth <- max(nchar(rnames, type = "width"), nchar(RowData, type = "width")) + 1
    totalwidth <- rlabwidth + (sum(clabwidth + 3))

    # Abbreviate row and column labels. Withdraw one char of the newlimit
    # label until the rows fit in the screen
    while(totalwidth > availablewidth){
        newlimit <- max(c(rlabwidth - 1, clabwidth)) - 1
        # FIXME: The algorithm should consider the minColWd of each column
        if(newlimit < minColWd)
            break

        rnames <- sapply(rnames, Abbrev1, newlimit)
        cnames <- sapply(cnames, Abbrev1, newlimit)
        RowData <- Abbrev1(RowData, newlimit)

        clabwidth <- nchar(cnames, type = "width")
        clabwidth[clabwidth < minColWd] <- minColWd
        rlabwidth <- max(nchar(rnames, type = "width"), nchar(RowData, type = "width")) + 1
        totalwidth <- rlabwidth + (sum(clabwidth + 3))
    }

    minColWd <- apply(nt, 2, function(x) max(nchar(x, type = "width")))
    clabwidth <- apply(cbind(clabwidth, minColWd), 1, max)
    dashedline <- rep("-", sum(clabwidth) + 3 * nc + rlabwidth)
    ddashedline <- gsub("-", "=", dashedline)

    # Calculate horizontal line locations
    if(x$total.c && !is.na(x$prop.col)[1])
        nrnt <- nrow(nt) - 2
    else if(x$total.c)
        nrnt <- nrow(nt) - 1
    else
        nrnt <- nrow(nt)
    n <- nrnt / nrow(tab)
    idxh <- seq(n+1, nrnt+1, n)
    # idxh <- idxh[idxh < nrow(nt)] # necessary when total.c = FALSE

    ## Print table cells
    cat("\n", ddashedline, "\n", sep = "")
    if(ColData != "")
        cat(formatC(" ", width = rlabwidth), "   ", ColData, "\n", sep = "", collapse = "")
    if(RowData == "")
        cat(formatC(" ", width = rlabwidth))
    else
        cat(formatC(RowData, width = rlabwidth, format = "s", flag = "-"))
    for(j in 1:nc)
        cat("  ", formatC(cnames[j], width = clabwidth[j]))
    cat("\n", dashedline, "\n", sep = "")
    for(i in 1:nr){
        if(i %in% idxh)
            cat(dashedline, "\n", sep = "")
        cat(formatC(rnames[i], width = rlabwidth, format = "s", flag = "-"), sep = "")
        if(x$percent && (prop.r || prop.c || prop.t)){
            for(j in 1:nc)
                if(grepl("%", nt[i, j]))
                    cat("   ", formatC(nt[i, j], width = clabwidth[j]), sep = "")
                else
                    cat("  ", formatC(nt[i, j], width = clabwidth[j]), " ", sep = "")
        } else {
            for(j in 1:nc)
                cat("  ", formatC(nt[i, j], width = clabwidth[j]))
        }
        cat("\n")
    }
    cat(ddashedline, "\n", sep = "")


    ## Print Statistics
    if (chisq)
    {
        cat("\n")
        cat(gettext("Statistics for All Table Factors", domain = "R-descr"),
            "\n\n", sep="")

        cat(CST$method, "\n")
        cat("------------------------------------------------------------\n")
        fp <- format.pval(CST$p.value, digits = dgts$others)
        pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
        cat(gettext("Chi^2 =", domain = "R-descr"), CST$statistic,
            "    ", gettext("d.f. =", domain = "R-descr"), CST$parameter,
            "    ", pv, "\n\n")

        if (all(dim(tab) == 2))
        {
            cat(CSTc$method, "\n")
            cat("------------------------------------------------------------\n")
            fp <- format.pval(CSTc$p.value, digits = dgts$others)
            pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
            cat(gettext("Chi^2 =", domain = "R-descr"), CSTc$statistic,
                "    ", gettext("d.f. =", domain = "R-descr"), CSTc$parameter,
                "    ", pv, "\n")
        }
    }

    ## Print McNemar tests
    if (is.na(McN[1]) == FALSE)
    {
        cat(rep("\n", 2))
        cat(McN$method, "\n")
        cat("------------------------------------------------------------\n")
        fp <- format.pval(McN$p.value, digits = dgts$others)
        pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
        cat(gettext("Chi^2 =", domain = "R-descr"), McN$statistic,
            "    ", gettext("d.f. =", domain = "R-descr"), McN$parameter,
            "    ", pv, "\n\n")

        if (is.na(McNc[1]) == FALSE)
        {
            cat(McNc$method, "\n")
            cat("------------------------------------------------------------\n")
            fp <- format.pval(McNc$p.value, digits = dgts$others)
            pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
            cat(gettext("Chi^2 =", domain = "R-descr"), McNc$statistic,
                "    ", gettext("d.f. =", domain = "R-descr"), McNc$parameter,
                "    ", pv, "\n")
        }
    }

    ## Pint Fisher Tests
    if (fisher)
    {
        cat(rep("\n", 2))

        cat(gettext("Fisher's Exact Test for Count Data", domain = "R-descr"))
        cat("\n------------------------------------------------------------\n")

        if (all(dim(tab) == 2))
        {
            cat(gettext("Sample estimate odds ratio:", domain = "R-descr"), FTt$estimate, "\n\n")

            cat(gettext("Alternative hypothesis: true odds ratio is not equal to 1",
                    domain = "R-descr"), "\n")
            fp <- format.pval(FTt$p.value, digits = dgts$others)
            pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
            cat(pv, "\n")
            cat(gettextf("95%s confidence interval:", "%", domain = "R-descr"), FTt$conf.int, "\n\n")

            cat(gettext("Alternative hypothesis: true odds ratio is less than 1",
                    domain = "R-descr"), "\n")
            fp <- format.pval(FTl$p.value, digits = dgts$others)
            pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
            cat(pv, "\n")
            cat(gettext("95%s confidence interval:", "%", domain = "R-descr"), FTl$conf.int, "\n\n")

            cat(gettext("Alternative hypothesis: true odds ratio is greater than 1",
                    domain = "R-descr"), "\n")
            fp <- format.pval(FTg$p.value, digits = dgts$others)
            pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
            cat(pv, "\n")
            cat(gettext("95%s confidence interval:", "%", domain = "R-descr"), FTg$conf.int, "\n\n")
        }
        else
        {
            cat(gettext("Alternative hypothesis: two.sided", domain = "R-descr"), "\n")
            fp <- format.pval(FTt$p.value, digits = dgts$others)
            pv <- paste("p", if(substr(fp, 1L, 1L) == "<") fp else paste("=", fp))
            cat(pv, "\n")
        }
    } ## End Of If(Fisher) Loop

    #  cat(rep("\n", 2))

    if(format == "SPSS"){
        if (any(dim(tab) >= 2) & any(chisq, mcnemar, fisher))
        {
            MinExpF = min(CST$expected)
            cat("       ", gettext("Minimum expected frequency:", domain = "R-descr"), MinExpF, "\n")
            NMinExpF = length(CST$expected[which(CST$expected<5)])
            if (NMinExpF > 0)
            {
                NCells = length(CST$expected)
                cat(gettext("Cells with Expected Frequency < 5:", domain = "R-descr"),
                    " ", NMinExpF, " ", gettext("of", domain = "R-descr"), " ",
                    NCells, " (", 100*NMinExpF/NCells, "%)\n", sep = "")
            }
            cat("\n")

        } ## End of if (any(dim(tab)...))
    }
    return(invisible(x))
}

as.data.frame.CrossTable <- function(x, ...) as.data.frame(x$tab, ...)


# Needed by tableStyles() of odfWeave package:
dim.CrossTable <- function(x){
    dim(x$tab) + 1
}

# From Hmisc::wtd.var
wtd.sd <- function(x, weights)
{
    xbar <- sum(weights * x)/sum(weights)
    sqrt(sum(weights * ((x - xbar)^2))/(sum(weights) - 1))
}


compmeans <- function(x, f, w, sort = FALSE, maxlevels = 60,
                      user.missing, missing.include = FALSE,
                      plot = getOption("descr.plot"),
                      relative.widths = TRUE, col = "lightgray",
                      warn = getOption("descr.warn"), ...)
{
    row.label <- attr(f, "label")
    column.label <- attr(x, "label")
    row.name <- deparse(substitute(f))
    column.name <- deparse(substitute(x))

    f.name <- deparse(substitute(f))
    n.name <- deparse(substitute(x))
    lf <- length(f)
    lx <- length(x)
    if (lf != lx) {
        msg <- paste(f.name, gettext("and", domain = "R-descr"), n.name,
                     gettext("have different lengths", domain = "R-descr"))
        stop(msg)
    }

    if (is.factor(f) == FALSE) {
        f <- factor(f)
        nl <- length(levels(f))
        if (nl > maxlevels) {
            msg <- paste(f.name,
                         gettext("was converted into a factor, but the new variable had too many levels",
                                 domain = "R-descr"))
            stop(msg)
        }
        if(warn){
            wmsg <- paste(gettext("Warning:", domain = "R-descr"), " \"", f.name,
                          "\" ", gettext("was converted into factor!",
                                         domain = "R-descr"), sep = "")
            warning(wmsg)
        }
    } else{
        class(f) <- "factor"
    }
    if(!missing(user.missing)){
        user.missing <- paste("^", user.missing, "$", sep = "")
        flevels <- levels(f)
        for(lev in user.missing){
            if(length(grep(lev, flevels))){
                idx <- grep(lev, as.character(f)) 
                if(length(idx))
                    f[idx] <- NA
            }
        }
        f <- factor(f)
    }
    if(missing.include)
        f <- no.drop.levels(f)

    if(is.numeric(x)){
        class(x) <- "numeric"
    }
    if (missing(w)) {
        wt <- rep(1, lf)
    } else {
        wt <- w
        lw <- length(w)
        if (lw != lf) {
            msg <- paste(f.name, gettext("and", domain = "R-descr"), "weight",
                         gettext("have different lengths.", domain = "R-descr"))
            stop(msg)
        }
    }
    if(is.numeric(wt)){
        class(wt) <- "numeric"
    }

    if (is.factor(x) == TRUE) {
        x <- as.numeric(x)
        if(warn){
            wmsg <- paste(gettext("Warning:", domain = "R-descr"), " \"", n.name, "\" ", 
                          gettext("was converted from factor into numeric!", domain = "R-descr"))
            warning(wmsg)
        }
    }
    has.w <- FALSE
    k <- grep(FALSE, (is.na(f) | is.na(x) | is.na(wt)))
    f <- f[k]
    x <- x[k]
    wt <- wt[k]
    lf2 <- length(f)
    if (lf > lf2 && warn) {
        cat("\n")
        msg <- gettext("rows with missing values dropped", domain = "R-descr")
        wmsg <- paste(lf - lf2, msg)
        warning(wmsg)
    }

    xwsum <- tapply(x * wt, f, sum)
    wsum <- tapply(wt, f, sum)
    xmean <- xwsum / wsum
    wsum <- round(wsum)
    wsd <- xmean

    nflevs <- length(levels(f))
    b <- split(data.frame(x, wt), f)
    wsd <- sapply(b, function(.df) wtd.sd(.df$x, .df$wt))

    width <- wsum
    l <- length(xmean)
    xmean[l+1] <- weighted.mean(x, wt)
    wsum[l+1] <- round(sum(wt))
    wsd[l+1] <- wtd.sd(x, wt)
    tab <- cbind(xmean, wsum, wsd)
    tabrn <- rownames(tab)
    tabrn[l+1] <- gettext("Total", domain = "R-descr")
    rownames(tab) <- tabrn
    colnames(tab) <- c(gettext("Mean", domain = "R-descr"),
                       gettext("N", domain = "R-descr"),
                       gettext("Std. Dev.", domain = "R-descr"))
    if(sort){
        len <- length(xmean)
        len1 <- len - 1
        ordl <- order(xmean[1:len1]) # Do not sort the "Total"
        tab <- tab[c(ordl, len),]
        width <- width[ordl]
        f <- factor(as.numeric(f), levels = ordl, labels = levels(f)[ordl])
    }
    attr(tab, "row.name") <- row.name
    attr(tab, "column.name") <- column.name
    attr(tab, "row.label") <- row.label
    attr(tab, "column.label") <- column.label
    attr(tab, "maxlevels") <- maxlevels
    attr(tab, "col") <- col
    if(relative.widths)
        attr(tab, "width") <- width
    else
        attr(tab, "width") <- rep(1, length(width))

    # Add attributes to plot the object:
    attr(tab, "x") <- x
    attr(tab, "f") <- f
    if(!missing(w))
        attr(tab, "wt") <- wt
    class(tab) <- c("meanscomp", "matrix")

    if(plot)
        plot.meanscomp(tab, ...)
    tab
}

print.meanscomp <- function(x, ...)
{
    rlab <- attr(x, "row.label")
    clab <- attr(x, "column.label")

    if(is.null(rlab))
        rlab <- attr(x, "row.name")
    if(is.null(clab))
        clab <- attr(x, "column.name")

    # 'domain' is necessary because this function is not exported to
    # 'descr' namespace.
    msg1 <- gettext("Mean value of", domain = "R-descr")
    msg2 <- gettext("according to", domain = "R-descr")
    lwd <- getOption("width")
    msg <- paste(msg1, ' "', clab, '" ', msg2, ' "', rlab, '"', sep = "")

    # Break the label string if it is too large:
    if(nchar(msg, type = "width") < lwd){
        cat(msg, "\n", sep = "")
    } else {
        if((nchar(msg1, type = "width") + nchar(clab, type = "width")) < lwd) {
            msg <- paste(msg1, ' "', clab, '" ', sep = "")
            if((nchar(msg, type = "width") + nchar(msg2, type = "width")) < lwd) {
                cat(msg, msg2, '\n', '"', rlab, '"', '\n', sep = "")
            } else {
                cat(msg, "\n", sep = "")
                if((nchar(msg2, type = "width") + nchar(rlab, type = "width")) < (lwd - 1)){
                    cat(msg2, ' "', rlab, '"\n', sep = "")
                } else {
                    cat(msg2, '\n"', rlab, '"\n', sep = "")
                }
            }
        } else {
            cat(msg1, '\n"', clab, '"\n', sep = "")
            if((nchar(msg2, type = "width") + nchar(rlab, type = "width")) < (lwd - 1)){
                cat(msg2, ' "', rlab, '"\n', sep = "")
            } else {
                cat(msg2, '\n"', rlab, '"\n', sep = "")
            }
        }
    }
    attr(x, "row.name") <- NULL
    attr(x, "column.name") <- NULL
    attr(x, "row.label") <- NULL
    attr(x, "column.label") <- NULL
    attr(x, "maxlevels") <- NULL
    attr(x, "x") <- NULL
    attr(x, "f") <- NULL
    attr(x, "wt") <- NULL
    attr(x, "col") <- NULL
    attr(x, "width") <- NULL
    class(x) <- "matrix"
    print(x, ...)
    return(invisible(NULL))
}

plot.meanscomp <- function(x, xlab, ylab, width, col, ...)
{
    if(missing(xlab))
        xlab <- attr(x, "row.label")
    if(missing(ylab))
        ylab <- attr(x, "column.label")
    if(is.null(xlab))
        xlab <- attr(x, "row.name")
    if(is.null(ylab))
        ylab <- attr(x, "column.name")

    if(missing(width))
        width <- attr(x, "width")
    if(missing(col))
        col <- attr(x, "col")
    maxlevels <- attr(x, "maxlevels")
    v <- attr(x, "x")
    f <- attr(x, "f")
    w <- attr(x, "wt")

    if(!is.factor(f))
        stop(gettext("f is not a factor.", domain = "R-descr"))
    if(length(levels(f)) > maxlevels)
        stop(gettext("Number of levels of \"f\" is higher than maxlevels.",
                     domain = "R-descr"))

    if(is.null(w)){
        boxplot(v ~ f, ylab = ylab, xlab = xlab, width = width, col = col, ...)
    } else {
        d <- data.frame(v, f, w)
        dd <- split(d, f)
        zz <- lapply(dd, function(d) spBwplotStats(d$v, d$w))
        z <- zz[[1]]
        z$nzero <- NULL
        z <- unclass(z)
        z$group <- rep(1, length(z$out))
        z$names <- levels(f)
        for(i in 2:length(zz)){
            z$stats <- cbind(z$stats, zz[[i]]$stats)
            z$n <- c(z$n, zz[[i]]$n)
            z$out <- c(z$out, zz[[i]]$out)
            z$group <- c(z$group, rep(i, length(zz[[i]]$out)))
        }
        bxp(z, ylab = ylab, xlab = xlab, width = width, boxfill = col, ...)
        return(invisible(z))
    }
}

file.head <- function(file, n = 6, truncate.cols = TRUE){
    lns <- readLines(file, n = n)
    lns <- gsub("\t", "\\\\t", lns)
    if(truncate.cols){
        try(file.head.lns <- substr(lns, 1, getOption("width") - 1), silent = TRUE)
        if(!exists("file.head.lns")){
            Encoding(lns) <- "bytes"
            file.head.lns <- substr(lns, 1, getOption("width") - 1)
        }
    }
    cat(file.head.lns, sep = "\n")
}

fwf2csv <- function(fwffile, csvfile, names, begin, end, verbose = getOption("verbose"))
{
    # Check for errors
    ncols = length(names)
    if(length(begin) != ncols || length(end) != ncols)
	stop("The vectors \"names\", \"begin\" and \"end\" must have the same length.")
    if(file.exists(fwffile) == FALSE){
	msg <- paste(gettext("File not found:", domain = "R-descr"), fwffile)
	stop(msg)
    }

    csvfile <- path.expand(csvfile)
    fwffile <- path.expand(fwffile)

    .C(realfwf2csv,
	as.character(fwffile),
	as.character(csvfile),
	as.character(names),
	as.integer(begin),
	as.integer(end),
	ncols,
        as.logical(verbose))

    return (invisible(NULL))
}

data.frame2txt <- function(x, datafile = "x.txt",
                           r.codefile = "x.R",
                           sps.codefile = "x.sps",
                           df.name = "x",
                           user.missing){
  x.names <- names(x)

  systemfile <- sub("\\....$", "", datafile)

  sink(r.codefile)
  cat(df.name, ' <- read.delim("', datafile, '", quote = "", as.is = TRUE)\n\n',
      sep = "")
  for(column in x.names){
    xx <- x[[column]]
    if(is.factor(xx)){
      xx.levels <- gsub('"', '\\\\"', levels(xx))
      n.levels <- length(xx.levels)
      cat(df.name, "$", column, " <- factor(", df.name, "$", column,
          ", levels = 1:", n.levels, ',\n  labels = c("', sep = "")
      cat(xx.levels[1], '"', sep = "")
      if(n.levels > 1) cat(", ")
      i <- 2
      len <- 2
      while(i < n.levels){
	len <- len + nchar(xx.levels[i]) + 4
	if(len > 80){
	  cat("\n    ")
	  len <- nchar(xx.levels[i]) + 6
	}
	cat('"', xx.levels[i], '", ', sep = "")
	i <- i + 1
      }
      if(len > 80) cat("\n  ")
      if(n.levels > 1) cat('"', xx.levels[n.levels], '"', sep = "")
      cat("))\n")
    }
  }
  for(column in x.names){
    xx <- x[[column]]
    xx.label <- attr(xx, "label")
    if(!is.null(xx.label)){
      cat("attr(", df.name, "$", column, ', "label") <- "', xx.label,
          '"\n', sep = "")
    }
  }
  cat("save(", df.name, ", file = \"", systemfile, ".RData\")\n", sep = "")
  sink()

  sink(sps.codefile)
  cat("GET DATA\n")
  cat("  /TYPE=TXT\n")
  cat("  /FILE='", datafile, "'\n", sep = "")
  cat("  /DELCASE=LINE\n")
  cat("  /DELIMITERS=\"\\t\"\n")
  cat("  /ARRANGEMENT=DELIMITED\n")
  cat("  /FIRSTCASE=2\n")
  cat("  /VARIABLES=\n")
  for(column in x.names){
    cat("  ", column, " ", sep = "")
    xx <- x[[column]]
    if(is.character(xx)){
        mnc <- max(nchar(xx), na.rm = TRUE)
        if(mnc == 0)
            mnc <- 1
        cat("A", mnc, "\n", sep = "")
    } else {
        if(is.logical(xx))
            xx <- as.numeric(xx)
        if(is.factor(xx)){
            nlevs <- length(levels(xx))
            if(nlevs < 10) cat("F1.0\n")
            else if(nlevs > 9 && nlevs < 100) cat("F2.0\n")
            else if(nlevs > 99) cat("F3.0\n")
        } else if(is.numeric(xx)){
            if(sum(grepl("(chron|dates|times)", class(xx))) > 0){
                cat("A", max(nchar(as.character(xx)), na.rm = TRUE), "\n", sep = "")
            } else {
                cat("F", max(nchar(as.character(xx)), na.rm = TRUE), ".0\n", sep = "")
            }
        } else {
            cat("error: undefined type\n")
        }
    }
  }
  cat("  .\n")
  cat("EXECUTE.\n\n")

  for(column in x.names){
    xx <- x[[column]]
    xx.label <- attr(xx, "label")
    if(!is.null(xx.label))
      cat("VARIABLE LABELS ", column, ' "', xx.label, '" .\n', sep = "")
  }
  cat("\n")

  for(column in x.names){
    xx <- x[[column]]
    if(is.factor(xx)){
      cat("VALUE LABELS ", column, "\n", sep = "")
      xx.levels <- levels(xx)
      len <- length(xx.levels)
      for(i in 1:len){
	if(i < len){
	  cat("  ", i, ' "', xx.levels[i], '"\n', sep = "")
	} else {
	  cat("  ", i, ' "', xx.levels[i], '" .\n', sep = "")
	}
      }
      if(!missing(user.missing)){
	user.missing <- paste("^", user.missing, "$", sep = "")
	nmiss <- 0
	umiss <- numeric()
	i <- 1
	for(lmiss in user.missing){
	  idx <- grep(lmiss, xx.levels)
	  if(length(idx) == 1){
	    nmiss  <- nmiss + 1
	    umiss[i] <- idx
	    i <- i + 1
	  } else {
	    if(length(idx) > 1){
	      msg <- paste(gettext("Repeated labels in ", domain = "R-descr"),
                           column, ": ",  lmiss, sep = "")
	      stop(msg)
	    }
	  }
	}
	if(nmiss > 0){
	  cat("MISSING VALUES", column, "(")
	  cat(umiss, sep = ", ")
	  cat(").\n")
	}
      }
      cat("\n")
    }
  }
  cat("SAVE OUTFILE='", systemfile, ".sav'\n  /COMPRESSED.\n", sep = "")
  sink()

  for(column in x.names)
    if(is.factor(x[[column]]) || is.logical(x[[column]]))
        x[[column]] <- as.numeric(x[[column]])

  write.table(x, file = datafile, quote = FALSE, sep = "\t", col.names = TRUE,
    row.names = FALSE, na = "")
}

crosstab <- function(dep, indep, weight = NULL,
                     digits = list(expected = 1, prop = 3, percent = 1, others = 3),
                     max.width = NA, expected = FALSE, prop.r = FALSE,
                     prop.c = FALSE, prop.t = FALSE, prop.chisq = FALSE,
                     chisq = FALSE, fisher = FALSE, mcnemar = FALSE, resid = FALSE,
                     sresid = FALSE, asresid = FALSE, missing.include = FALSE,
                     drop.levels = TRUE, format = "SPSS", cell.layout = TRUE,
                     row.labels = !cell.layout,
                     percent = (format == "SPSS" && !row.labels),
                     total.r, total.c,
                     dnn = "label", xlab = NULL, ylab = NULL, main = "",
                     user.missing.dep, user.missing.indep,
                     plot = getOption("descr.plot"), ...)
{
    if(missing(dep))
        stop("The argument 'dep' (dependent variable) is missing.")
    if(missing(indep))
        stop("The 'indep' (independent variable) is missing. Please, consider using either CrossTable() or freq().")

    if(length(dnn) == 1 && dnn == "label"){
        dimn <- c(deparse(substitute(dep)), deparse(substitute(indep)))
        if(!is.null(attr(dep, "label")))
            dimn[1] <- attr(dep, "label")
        if(!is.null(attr(indep, "label")))
            dimn[2] <- attr(indep, "label")
        dnn <- dimn
    }
    if(is.null(dnn))
        dnn <- c(deparse(substitute(dep)), deparse(substitute(indep)))

    if(!missing(user.missing.indep)){
        user.missing.indep <- paste("^", user.missing.indep, "$", sep = "")
        ilevels <- levels(indep)
        for(lev in user.missing.indep){
            if(length(grep(lev, ilevels))){
                idx <- grep(lev, as.character(indep))
                if(length(idx))
                    indep[idx] <- NA
            }
        }
        indep <- factor(indep)
    }
    if(!missing(user.missing.dep)){
        user.missing.dep <- paste("^", user.missing.dep, "$", sep = "")
        dlevels <- levels(dep)
        for(lev in user.missing.dep){
            if(length(grep(lev, dlevels))){
                idx <- grep(lev, as.character(dep))
                if(length(idx))
                    dep[idx] <- NA
            }
        }
        dep <- factor(dep)
    }
    if(missing.include){
        dep <- no.drop.levels(dep)
        indep <- no.drop.levels(indep)
    }
    if(drop.levels){
        dep <- factor(dep)
        indep <- factor(indep)
    }
    if (is.null(weight)){
        tab <- table(dep, indep)
    } else {
        if(getOption("descr.round.xtabs"))
            tab <- round(xtabs(weight ~ dep + indep))
        else
            tab <- xtabs(weight ~ dep + indep)
    }
    names(dimnames(tab)) <- dnn

    if(!missing(total.r)){
        if(!is.logical(total.r))
            stop(gettext("total.r must be logical", domain = "R-descr"))
        if(missing(total.c))
            total.c <- total.r
    }
    if(!missing(total.c)){
        if(!is.logical(total.c))
            stop(gettext("total.c must be logical", domain = "R-descr"))
        if(missing(total.r))
            total.r <- total.c
    }
    if(missing(total.r) & missing(total.c))
        total.r <- total.c <- TRUE

    crosstb <- CrossTable(tab, digits = digits, max.width = max.width,
                          expected = expected, prop.r = prop.r,
                          prop.c = prop.c, prop.t = prop.t,
                          prop.chisq = prop.chisq, chisq = chisq,
                          fisher = fisher, mcnemar = mcnemar, resid = resid,
                          sresid = sresid, asresid = asresid,
                          missing.include = missing.include,
                          drop.levels = drop.levels, format = format, dnn = dnn,
                          cell.layout = cell.layout, row.labels = row.labels,
                          percent = percent, total.r = total.r,
                          total.c = total.c, xlab = xlab, ylab = ylab)

    if(plot == TRUE)
        plot.CrossTable(crosstb, ...)

    crosstb
}


plot.CrossTable <- function(x, xlab, ylab, main = "", col, inv.x = FALSE, inv.y = FALSE, ...)
{
    tabforplot <- t(x$tab)
    if(missing(xlab)){
        lab <- attr(x, "xlab")
        if(is.null(lab))
            xlab <- x$ColData
        else
            xlab <- lab
    }
    if(missing(ylab)){
        lab <- attr(x, "ylab")
        if(is.null(lab))
            ylab <- x$RowData
        else
            ylab <- lab
    }
    nxlev <- dim(tabforplot)[1]
    nylev <- dim(tabforplot)[2]
    if(missing(col)){
        col.min <- 0.9 - 0.25 * (nylev - 1)
        if(col.min < 0.3)
            col.min  <- 0.3
        col <- gray.colors(nylev, 0.9, col.min)
    }
    if(inv.x)
        tabforplot <- tabforplot[nxlev:1, ]
    if(inv.y)
        tabforplot <- tabforplot[, nylev:1]
    class(tabforplot) <- "table"
    if(length(grep("^color$", names(list(...)))) == 0)
        mosaicplot(tabforplot, main = main, xlab = xlab, ylab = ylab, col = col, ...)
    else
        mosaicplot(tabforplot, main = main, xlab = xlab, ylab = ylab, ...)
}

labels2R <- function(lfile, rfile, dfname = "b", echo = FALSE)
{
    if (missing(lfile)) 
        stop(gettext("The name of file with labels is required.", 
            domain = "R-descr"))
    if (missing(rfile)) 
        stop(gettext("The name of file with R code to is required.", 
            domain = "R-descr"))
    if (!is.character(lfile)) 
        stop("lfile must be of class character.")
    if (!is.character(rfile)) 
        stop("rfile must be of class character.")
    if (!is.character(dfname)) 
        stop("dfname must be of class character.")
    infile <- path.expand(lfile[1])
    outfile <- path.expand(rfile[1])
    if (!file.exists(infile)) {
        msg <- paste(gettext("File not found:", domain = "R-descr"), 
            lfile)
        stop(msg)
    }
    if (file.exists(outfile)) {
        unlink(outfile)
    }
    input <- readLines(infile)

    # The last line must be empty
    input <- c(input, "")

    nlines <- length(input)
    lnum <- 1
    while (lnum <= nlines) {
        cline <- input[lnum]
        if(echo)
            cat("[", lnum, "]", cline, "\n", sep = "")
        varname <- NULL
        varlab <- NULL
        lev <- NULL
        lab <- NULL
        if (cline != "" && grep("^[a-zA-Z]", cline) == 1) {
            varname <- sub("^([a-zA-Z0-9_\\.]*).*", "\\1", cline)
            if (grep(" ", cline) == 1) 
                varlab <- sub("(\\w|\\.|_)* (.*)", "\\2", cline)
            lnum <- lnum + 1
            cline <- input[lnum]
            if(echo)
                cat("[", lnum, "]", cline, "\n", sep = "")
            nlev = 0
            while (cline != "" && (grep("^[0-9]* ", cline) == 1 || grep("^-[0-9]* ", cline) == 1) && lnum <= nlines) {
                nlev <- nlev + 1
                lev[nlev] <- sub("^(-*[0-9]*) .*", "\\1", cline)
                lab[nlev] <- sub("^-*[0-9]* (.*)", "\\1", cline)
                if (lnum < nlines) {
                  lnum <- lnum + 1
                  cline <- input[lnum]
                  if(echo)
                      cat("[", lnum, "]", cline, "\n", sep = "")
                }
            }
            if (!is.null(lev)) 
                cat(dfname, "$", varname, " <- factor(", dfname, 
                  "$", varname, ",\n    levels = c(", paste(lev, 
                    collapse = ", "), "),\n    labels = c(\"", 
                  paste(lab, collapse = "\", \""), "\"))\n", 
                  sep = "", file = outfile, append = TRUE)
            if (!is.null(varlab)) 
                cat("attr(", dfname, "$", varname, ", \"label\") <- \"", 
                  varlab, "\"\n", sep = "", file = outfile, append = TRUE)
            cat("\n", file = outfile, append = TRUE)
        }
        if (lnum <= nlines)
            lnum <- lnum + 1
    }
    return(invisible(NULL))
}

\name{freq}
\alias{freq}
\title{Frequency table}
\description{
  Prints a frequency table of the selected object. Optionally, the frequency
  might be weighted.
}
\usage{
freq(x, w, user.missing, plot = getOption("descr.plot"), \dots)
}
\arguments{
  \item{x}{The factor from which the frequency of values is desired.}
  \item{w}{An optional vector for a weighted frequency table.}
  \item{user.missing}{Character vector, indicating what levels must be
    treated as missing values while calculating valid percents. Levels
    representing user missing values are not shown in the
    \code{\link[graphics]{barplot}}.}
  \item{plot}{Logical: if \code{TRUE} (default), a barplot is produced.
    You may put
   
    \code{options(descr.plot = FALSE)}
   
    in your \file{.Rprofile} to change the default function behavior.}
  \item{\dots}{Further arguments to be passed to \code{\link{plot.freqtable}}
    if \code{plot = TRUE}.}
}
\details{
  A column with cumulative percents are added to the frequency table if \code{x}
  is an \code{ordered factor}.
}
\value{
  A matrix with class \code{c("matrix", "freqtable")} with the attribute
  \code{"xlab"} which is a character string corresponding to either the
  attribute "label" of \code{x} or, if \code{x} does not have this attribute,
  the name of \code{x}. The returned object can be plotted, generating a
  \code{\link[graphics]{barplot}}.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}, based on function written by Dirk Enzmann}

\examples{
x <- c(rep(1, 100), rep(2, 120), rep(3, 10), rep(NA, 12))
w <- c(rep(1.1, 122), rep(0.9, 120))
x <- factor(x, levels = c(1, 2, 3),
            labels = c("No", "Yes", "No answer"))
attr(x, "label") <- "Do you agree?"

freq(x, y.axis = "percent")
f <- freq(x, w, user.missing = "No answer", plot = FALSE)
f
plot(f)

# If the decimal separator in your country is a comma:
# options(OutDec = ",")
library(xtable)
print(xtable(f))
}

\name{xtable.CrossTable}
\alias{xtable.CrossTable}
\title{CrossTable method for xtable}
\description{
  The method creates an object of class xtable.
}
\usage{
\method{xtable}{CrossTable}(x, caption = NULL, label = NULL,
       align = NULL, digits = NULL, display = NULL,
       auto = FALSE, multirow = FALSE, hline = FALSE, \dots)
}

\arguments{
  \item{x}{A object of class CrossTable.}
  \item{caption}{See \code{\link[xtable]{xtable}}.}
  \item{label}{See \code{\link[xtable]{xtable}}.}
  \item{align}{See \code{\link[xtable]{xtable}}.}
  \item{display}{See \code{\link[xtable]{xtable}}.}
  \item{digits}{See \code{\link[xtable]{xtable}}.}
  \item{auto}{See \code{\link[xtable]{xtable}}.}
  \item{multirow}{A logical value indicating whether the command
    \verb{\multirow} should be added to the table. See the Details section
    below.}
  \item{hline}{A logical value indicating whether the command
    \verb{\hline} should be added to the table. See the Details section
    below.}
  \item{\dots}{Further arguments to be passed to \code{\link[base]{format}} or
    to replace arguments previously passed to \code{\link{CrossTable}}.}
}

\details{
  If either \code{multirow} or \code{hline} is \code{TRUE}, the
  \code{sanitize.text.function} argument of \code{\link[xtable]{print.xtable}}
  must be defined. You will also have to add \verb{\usepackage\{multirow\}} to
  your Rnoweb document.  See the \code{Example} section of
  \code{\link{crosstab}}.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\seealso{\code{\link{CrossTable}}, \code{\link{crosstab}},
  \code{\link[xtable]{print.xtable}}.}

\name{forODFTable}
\alias{forODFTable}

\title{Convert an object of class CrossTable into a matrix for odfTable}
\description{The function converts an object of class CrossTable into a matrix
  to be printed by \samp{odfTable()} of \pkg{odfWeave} package.}
\usage{
forODFTable(x, digits = 1, \dots) 
}
\arguments{
  \item{x}{A object of class \samp{CrossTable}.}
  \item{digits}{See \link[base]{round}.}
  \item{\dots}{Optional arguments passed to \link[base]{format}.}
}
\value{
  A matrix.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}.}

\seealso{\link{CrossTable}}

\examples{
\dontrun{
library(odfWeave)
data(infert, package = "datasets")
x <- crosstab(infert$education, infert$induced, expected = TRUE)

# Use the function directly:
odfTable(forODFTable(x))

# Create a method for odfTable:
odfTable.CrossTable <- function(x) odfTable(forODFTable(x))
odfTable(x)
methods(odfTable)
}
}

\name{crosstab}
\alias{crosstab}
\title{Cross tabulation with mosaic plot}
\description{
  This function is a wrapper for \code{\link{CrossTable}}, adding a
  mosaic plot and making it easier to do a weighted cross-tabulation.
}
\usage{
crosstab(dep, indep, weight = NULL,
         digits = list(expected = 1, prop = 3, percent = 1, others = 3),
         max.width = NA,
         expected = FALSE, prop.r = FALSE, prop.c = FALSE, prop.t = FALSE,
         prop.chisq = FALSE, chisq = FALSE, fisher = FALSE, mcnemar = FALSE,
         resid = FALSE, sresid = FALSE, asresid = FALSE,
         missing.include = FALSE, drop.levels = TRUE, format = "SPSS",
         cell.layout = TRUE, row.labels = !cell.layout,
         percent = (format == "SPSS" && !row.labels),
         total.r, total.c, dnn = "label", xlab = NULL,
         ylab = NULL, main = "", user.missing.dep, user.missing.indep,
         plot = getOption("descr.plot"), \dots)
}

\arguments{
  \item{dep, indep}{Vectors in a matrix or a dataframe. \code{dep} should be
    the dependent variable, and \code{indep} should be the independent one.}
  \item{weight}{An optional vector for a weighted cross tabulation.}
  \item{digits}{See \code{\link{CrossTable}}.}
  \item{max.width}{See \code{\link{CrossTable}}.}
  \item{expected}{See \code{\link{CrossTable}}.}
  \item{prop.r}{See \code{\link{CrossTable}}.}
  \item{prop.c}{See \code{\link{CrossTable}}.}
  \item{prop.t}{See \code{\link{CrossTable}}.}
  \item{prop.chisq}{See \code{\link{CrossTable}}.}
  \item{chisq}{See \code{\link{CrossTable}}.}
  \item{fisher}{See \code{\link{CrossTable}}.}
  \item{mcnemar}{See \code{\link{CrossTable}}.}
  \item{resid}{See \code{\link{CrossTable}}.}
  \item{sresid}{See \code{\link{CrossTable}}.}
  \item{asresid}{See \code{\link{CrossTable}}.}
  \item{missing.include}{See \code{\link{CrossTable}}.}
  \item{drop.levels}{See \code{\link{CrossTable}}.}
  \item{format}{See \code{\link{CrossTable}}.}
  \item{cell.layout}{See \code{\link{CrossTable}}.}
  \item{row.labels}{See \code{\link{CrossTable}}.}
  \item{percent}{See \code{\link{CrossTable}}.}
  \item{total.r}{See \code{\link{CrossTable}}.}
  \item{total.c}{See \code{\link{CrossTable}}.}
  \item{dnn}{See \code{\link{CrossTable}}. If \code{dnn = "label"}, then the
    \samp{"label"} attribute of \samp{dep} and \samp{indep} will be used as
    the dimension names.}
  \item{xlab}{See \code{\link[graphics]{plot.default}}.}
  \item{ylab}{See \code{\link[graphics]{plot.default}}.}
  \item{main}{An overall title for the plot (see
    \code{\link[graphics]{plot.default}} and \code{\link[graphics]{title}}).}
  \item{user.missing.dep}{An optional character vector with the levels of
    \code{dep} that should be treated as missing values.}
  \item{user.missing.indep}{An optional character vector with the levels of
    \code{indep} that should be treated as missing values.}
  \item{plot}{Logical: if \code{TRUE} (default), a mosaic plot is produced.
    You may put

    \code{options(descr.plot = FALSE)}

    in your \file{.Rprofile} to change the default function behavior.}
  \item{\dots}{Further arguments to be passed to
    \code{\link[graphics]{mosaicplot}}.}
}

\details{
  \code{crosstab} invokes the \code{\link{CrossTable}} with all boolean
  options set to \code{FALSE} and \code{"SPSS"} as the default \code{format}
  option. The returned CrossTable object can be plotted as a
  \code{\link[graphics]{mosaicplot}}. Note that the gray scale colors used by
  default in the mosaic plot do not have any statistical meaning. The colors
  are used only to ease the plot interpretation.

  Differently from \code{\link{CrossTable}}, this function requires both
  \code{dep} and \code{indep} arguments. If you want an univariate tabulation,
  you should try either \code{\link{CrossTable}} or \code{\link{freq}}.

  By default, if \code{weight} has decimals, the result of \code{xtabs} is
  rounded before being passed to \code{\link{CrossTable}}. If you prefer that
  the results are not rounded, add to your code:

  \code{options(descr.round.xtabs = FALSE)}
}

\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\seealso{\code{\link{CrossTable}}, \code{\link{plot.CrossTable}},
  \code{\link{xtable.CrossTable}}.}

\examples{
educ <- sample(c(1, 2), 200, replace = TRUE, prob = c(0.3, 0.7))
educ <- factor(educ, levels = c(1, 2), labels = c("Low", "High"))
opinion <- sample(c(1, 2, 9), 200, replace = TRUE,
                 prob = c(0.4, 0.55, 0.05))
opinion <- factor(opinion, levels = c(1, 2, 9),
                 labels = c("Disagree", "Agree", "Don't know"))
attr(educ, "label") <- "Education level"
attr(opinion, "label") <- "Opinion"
weight <- sample(c(10, 15, 19), 200, replace = TRUE)

crosstab(opinion, educ, xlab = "Education", ylab = "Opinion")
ct <- crosstab(opinion, educ, weight,
               dnn = c("Opinion", "Education"),
               user.missing.dep = "Don't know",
               expected = TRUE, prop.c = TRUE, prop.r = TRUE,
               plot = FALSE)
ct
plot(ct, inv.y = TRUE)

# Get the table of observed values as an object of class "table"
tab <- ct$tab
class(tab)
tab

# Get the complete cross table as "matrix"
complete.tab <- descr:::CreateNewTab(ct)
class(complete.tab)
complete.tab

## xtable support
library(xtable)

# Print ugly table
print(xtable(ct))

# Print pretty table
# Add to the preamble of your Rnoweb document:
# \usepackage{booktabs}
# \usepackage{multirow}
# \usepackage{dcolumn}
# \newcolumntype{d}{D{.}{.}{-1}}
print(xtable(ct, align = "llddd", multirow = TRUE, hline = TRUE,
             row.labels = TRUE, percent = FALSE,
             caption = "Opinion according to level of education"),
      booktabs = TRUE, include.rownames = FALSE,
      sanitize.text.function = function(x) x)
}

\name{fwf2csv}
\alias{fwf2csv}
\title{Fast conversion of a fwf file into a csv one}
\description{
  Convert fixed width formated file into a tab separated one.
}
\usage{
fwf2csv(fwffile, csvfile, names, begin, end,
        verbose = getOption("verbose"))
}
\arguments{
  \item{fwffile}{The fixed width format file.}
  \item{csvfile}{The csv file to be created. The fields will be separated by tab
    characters and there will be no quotes around strings.}
  \item{names}{A character vector with column names.}
  \item{begin}{A numeric vector with the begin offset of values in the fixed
    width format file.}
  \item{end}{A numeric vector with the end offset of values in the fixed width
    format file.}
  \item{verbose}{Logical: if \code{TRUE} a message about the number of saved
    lines is printed.}
}

\value{
  NULL.
}

\details{

  The return value is NULL, but \code{cvsfile} is created if the function is
  successful. The file is a text table with fields separated by tabular
  characters without quotes around the strings.
  
  This function is useful if you have a very big fixed width formated file to
  read and \link[utils]{read.fwf} would be too slow. The function that does
  the real job is very fast because it is written in C, and the use of RAM is
  minimum.

}

\seealso{For an efficient way of reading a csv file, see the function
  \samp{fread()} from \pkg{data.table} package.}

\examples{
txt_file <- tempfile()
csv_file <- tempfile()

# Column:     12345678901234567
writeLines(c("CE  1  11M43 2000",
             "CE  1  12F40 1800",
             "CE  1  13F 9    0",
             "CE  1  13M 6    0",
             "CE  2  21F36 1200",
             "CE  2  23M 6    0",
             "BA  1  11M33 2100",
             "BA  1  12F34 2300",
             "BA  1  13M10    0",
             "BA  1  13F 7    0",
             "BA  2  21F26 3600",
             "BA  2  22M27 3200",
             "BA  2  23F 2    0"),
           con = txt_file)

tab <- rbind(c("state",   1,  2),
             c("municp",  3,  5),
             c("house",   6,  8),
             c("cond",    9,  9),
             c("sex",    10, 10),
             c("age",    11, 12),
             c("income", 13, 17))

fwf2csv(txt_file, csv_file,
        names = tab[, 1],
        begin = as.numeric(tab[, 2]),
        end = as.numeric(tab[, 3]))
d <- read.table(csv_file, header = TRUE,
                 sep = "\t", quote = "")
d$cond <- factor(d$cond, levels = c(1, 2, 3),
                 labels = c("Reference", "Spouse", "Child"))
d$sex <- factor(d$sex)
d
}

\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\name{descr}
\alias{descr}
\title{Summary of an object}
\description{
  Wrapper for the function \code{summary} of \pkg{base} package, including
  information about variable label.  The function prints the \code{label}
  attribute of the object and, then, invokes \code{summary(object)}. If the object
  is a data frame, the function prints the \code{label} and invokes
  \code{summary} for each variable in the data frame.
}
\usage{
descr(x)
}
\arguments{
  \item{x}{The object to be described.}
}

\value{
  Null.
}

\author{Jakson Aquino \email{jalvesaq@gmail.com}}

\seealso{\code{\link[base]{summary}}}

\name{plot.freqtable}
\alias{plot.freqtable}
\title{Bar plot from object of class freqtable}
\description{
  This function receives a \code{freqtable} object as its main
  argument and produces a barplot.
}
\usage{
\method{plot}{freqtable}(x, y.axis = "count", \dots)
}

\arguments{
  \item{x}{A object of class \code{freqtable}.}
  \item{y.axis}{Character string, indicating what variable to use in the y
    axis, "count" or "percent", when plotting the frequency table.}
  \item{\dots}{Further arguments to be passed to \code{\link[graphics]{barplot}}.}
}

\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\name{toUTF8}
\alias{toUTF8}

\title{Conversion to UTF-8 encoding}
\description{Converts the encoding of some attributes of an object to UTF-8}
\usage{
toUTF8(x, from = "WINDOWS-1252") 
}
\arguments{
  \item{x}{A R object, usually a variable of a data frame or a data frame.}
  \item{from}{A string indicating the original encoding. Common values are
    \code{"LATIN1"} and \code{"WINDOWS-1252"}. Type \code{iconvlist()} for the
    complete list of available encodings.}
}
\details{
  The function converts the attribute \code{label} of \code{x} from the
  specified encoding into UTF-8. If \code{x} is a factor, the levels are
  converted as well. If \code{x} is a data.frame, the function makes the
  conversions in all of its variables.
}
\value{
  The object with its label and levels converted.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}.}

\seealso{\link[base]{iconv}}

\name{CrossTable}
\alias{CrossTable}
\title{Cross tabulation with tests for factor independence}
\description{
  An implementation of a cross-tabulation function with output
  similar to S-Plus crosstabs() and SAS Proc Freq (or SPSS format)
  with Chi-square, Fisher and McNemar tests of the independence
  of all table factors.
}
\usage{
CrossTable(x, y,
           digits = list(expected = 1, prop = 3, percent = 1, others = 3),
           max.width = NA, expected = FALSE,
           prop.r = TRUE, prop.c = TRUE, prop.t = TRUE,
           prop.chisq = TRUE, chisq = FALSE, fisher = FALSE,
           mcnemar = FALSE, resid = FALSE, sresid = FALSE,
           asresid = FALSE, missing.include = FALSE,
           drop.levels = TRUE, format = c("SAS","SPSS"),
           dnn = NULL, cell.layout = TRUE,
           row.labels = !cell.layout,
           percent = (format == "SPSS" && !row.labels),
           total.r, total.c, xlab = NULL, ylab = NULL, \dots)
}
\arguments{
  \item{x}{A vector or a matrix. If y is specified, x must be a vector.}
  \item{y}{A vector in a matrix or a dataframe.}
  \item{digits}{Named list with number of digits after the decimal point for
    four categories of statistics: expected values, cell proportions,
    percentage and others statistics. It can also be a numeric vector with a
    single number if you want the same number of digits in all statistics.}
  \item{max.width}{In the case of a 1 x n table, the default will be
    to print the output horizontally. If the number of columns exceeds
    max.width, the table will be wrapped for each successive increment
    of max.width columns. If you want a single column vertical table,
    set max.width to 1.}
  \item{prop.r}{If \code{TRUE}, row proportions will be included.}
  \item{prop.c}{If \code{TRUE}, column proportions will be included.}
  \item{prop.t}{If \code{TRUE}, table proportions will be included.}
  \item{expected}{If \code{TRUE}, expected cell counts from the
    \eqn{\chi^2}{Chi-Square} will be included.}
  \item{prop.chisq}{If \code{TRUE}, chi-square contribution of each cell will be
    included.}
  \item{chisq}{If \code{TRUE}, the results of a chi-square test will be
    printed after the table.}
  \item{fisher}{If \code{TRUE}, the results of a Fisher Exact test will
    be printed after the table}
  \item{mcnemar}{If \code{TRUE}, the results of a McNemar test will be printed
  after the table.}
  \item{resid}{If \code{TRUE}, residual (Pearson) will be included.}
  \item{sresid}{If \code{TRUE},  standardized residual will be included.}
  \item{asresid}{If \code{TRUE}, adjusted standardized residual will be
    included.}
  \item{missing.include}{If \code{TRUE}, then NA values, if present, are
    included as level \code{"NA"} of both x and y. You can change the new
    level label by setting the value of descr.na.replacement option. Example:
    \code{options(descr.na.replacement = "Missing")}.}
  \item{drop.levels}{If \code{TRUE}, then remove any unused factor levels.}
  \item{format}{Either SAS (default) or SPSS, depending on the type of
    output desired.}
  \item{dnn}{The names to be given to the dimensions in the result (the dimnames
    names).}
  \item{cell.layout}{If \code{TRUE}, print the cell layout.}
  \item{row.labels}{If \code{TRUE}, add labels to rows of calculated
    statistics.}
  \item{percent}{A logical value indicating whether to add the percentage
    symbol \samp{prop.r}, \samp{prop.c} and \samp{prop.t} if \samp{format} is
    \samp{"SPSS"}.}.
  \item{total.r}{If \code{TRUE}, print row totals.}
  \item{total.c}{If \code{TRUE}, print column totals.}
  \item{xlab}{A title for the x axis when plotting the CrossTable object (see
    \code{\link[graphics]{title}}). If missing, \code{dnn[1]} is used if not
    \code{NULL}.}
  \item{ylab}{A title for the y axis when plotting the CrossTable object (see
    \code{\link[graphics]{title}}). If missing, \code{dnn[2]} is used if not
    \code{NULL}.}
  \item{\dots}{Optional arguments passed to \code{\link[stats]{chisq.test}}.}
}
\details{
  A summary table will be generated with cell row, column and table
  proportions and marginal totals and proportions. Expected cell counts can be
  printed if desired. In the case of a 2 x 2 table, both corrected and
  uncorrected values will be included for appropriate tests. In the case of
  tabulating a single vector, cell counts and table proportions will be
  printed.

  Note 1: If 'x' is a vector and 'y' is not specified, no statistical tests
  will be performed, even if any are set to \code{TRUE}.

  Note 2: 'x' and 'y' labels will be truncated if the table is not going to
  fit to the screen, according to the value of \code{getOption("width")}.


  If both arguments \samp{total.c} and \samp{total.r} are missing, both will
  be \code{TRUE}. If only one of them is missing, the other will have the same
  value of the not missing one.
}
\value{
  A list of class \code{CrossTable} containing parameters used by the
  \code{print.CrossTable} method and the following components:

  tab:          An n by m matrix containing table cell counts.

  prop.row:     An n by m matrix containing cell row proportions.

  prop.col:     An n by m matrix containing cell column proportions.

  prop.tbl:     An n by m matrix containing cell table proportions.

  chisq:        Results from the Chi-Square test. A list with class 'htest'.
  See \code{\link[stats]{chisq.test}} for details.

  chisq.corr:   Results from the corrected Chi-Square test. A list with class
  'htest'. See \code{\link[stats]{chisq.test}} for details. ONLY included in
  the case of a 2 x 2 table.

  fisher.ts:    Results from the two-sided Fisher Exact test. A list with
  class 'htest'. See \code{\link[stats]{fisher.test}} for details. ONLY
  included if 'fisher' = TRUE.

  fisher.lt:    Results from the Fisher Exact test with HA = "less". A list
  with class 'htest'. See \code{\link[stats]{fisher.test}} for details. ONLY
  included if 'fisher' = TRUE and in the case of a 2 x 2 table.

  fisher.gt:    Results from the Fisher Exact test with HA = "greater". A list
  with class 'htest'. See \code{\link[stats]{fisher.test}} for details. ONLY
  included if 'fisher' = TRUE and in the case of a 2 x 2 table.

  mcnemar:      Results from the McNemar test. A list with class 'htest'. See
  \code{\link[stats]{mcnemar.test}} for details. ONLY included if 'mcnemar' =
  TRUE.

  mcnemar.corr: Results from the corrected McNemar test. A list with class
  'htest'. See \code{\link[stats]{mcnemar.test}} for details. ONLY included if
  'mcnemar' = TRUE and in the case of a 2 x 2 table.

  resid/sresid/asresid: Pearson Residuals (from chi-square tests).

}
\author{Jakson Aquino \email{jalvesaq@gmail.com} has splited the function
  \code{CrossTable} (from the package \code{gmodels}) in two:
  \code{CrossTable} and \code{print.CrossTable}. The \code{gmodels}'s function
  was developed by Marc Schwartz (original version posted to r-devel on Jul
  27, 2002. SPSS format modifications added by Nitin Jain based upon code
  provided by Dirk Enzmann).}

\seealso{\code{\link{crosstab}} (a wrapper to \samp{CrossTable} that makes it
  easier to do a weighted contingency table), \code{\link{plot.CrossTable}},
  \code{\link{forODFTable}}, \code{\link[base]{table}},
  \code{\link[base]{prop.table}}, \code{\link[stats]{xtabs}}.}

\examples{
# Simple cross tabulation of education versus prior induced
# abortions using infertility data
data(warpbreaks, package = "datasets")
ct <- CrossTable(warpbreaks$wool, warpbreaks$tension,
                 dnn = c("Wool", "Tension"))
data(esoph, package = "datasets")
ct <- CrossTable(esoph$alcgp, esoph$agegp, expected = TRUE,
                 chisq = FALSE, prop.chisq = FALSE,
                 dnn = c("Alcohol consumption", "Tobacco consumption"))
plot(ct, inv.y = TRUE)
print(ct)

# While printing the object, you can replace some (but not all)
# arguments previously passed to CrossTable
print(ct, format = "SPSS", cell.layout = FALSE, row.labels = TRUE)

# For better examples, including the use of xtable,
# see the documentation of crosstab().
}

\name{compmeans}
\alias{compmeans}
\title{Means of a numerical vector according to a factor}
\description{Calculates the means of a numerical vector according to a factor.}
\usage{
compmeans(x, f, w, sort = FALSE, maxlevels = 60,
          user.missing, missing.include = FALSE,
          plot = getOption("descr.plot"),
          relative.widths = TRUE, col = "lightgray",
          warn = getOption("descr.warn"), \dots)
}
\arguments{
  \item{x}{A numeric vector.}
  \item{f}{A factor.}
  \item{w}{Optional vector with weights.}
  \item{sort}{If \code{TRUE}, sorts the lines by the means values.}
  \item{maxlevels}{Maximum number of levels that \code{x} converted into factor
    should have.}
  \item{user.missing}{Character vector, indicating what levels of \code{f}
    must be treated as missing values.}
  \item{missing.include}{If \code{TRUE}, then NA values, if present in
    \code{f}, are included as level \code{"NA"}. You can change the new level
    label by setting the value of descr.na.replacement option. Example:
    \code{options(descr.na.replacement = "Missing")}.}
  \item{plot}{Logical: if \code{TRUE} (default), a boxplot is produced.
    You may put

    \code{options(descr.plot = FALSE)}

    in your \file{.Rprofile} to change the default function behavior.}
  \item{relative.widths}{If \code{TRUE}, the boxes widths will be proportional
    to the number of elements in each level of \code{f}.}
  \item{col}{Vector with the boxes colors.}
  \item{warn}{Warn if conversion from factor into numeric or from numeric into
    factor was performed and if missing values were dropped (default: \code{TRUE}).}
  \item{\dots}{Further arguments to be passed to either
    \code{\link[graphics]{boxplot}} (if \code{w} is missing) or
    \code{\link[graphics]{bxp}} (for \code{w} weighted boxplot).}
}
\value{
  A matrix with class \code{c("matrix", "meanscomp")} with labels attributes
  for \code{x} and \code{f}. The returned object can be plotted, generating
  a \code{\link[graphics]{boxplot}} of \code{x} grouped by \code{f}.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}, with code for weighted
  boxplots written by Stefan Kraft for simPopulation package.}

\seealso{\code{\link[graphics]{boxplot}}.}

\examples{
sex <- factor(c(rep("F", 900), rep("M", 900)))
income <- 100 * (rnorm(1800) + 5)
weight <- rep(1, 1800)
weight[sex == "F" & income > 500] <- 3
attr(income, "label") <- "Income"
attr(sex, "label") <- "Sex"
compmeans(income, sex, col = "lightgray", ylab = "income", xlab = "sex")
comp <- compmeans(income, sex, weight, plot = FALSE)
plot(comp, col = c("pink", "lightblue"), ylab = "income", xlab = "sex")

library(xtable)
# If the decimal separator in your country is a comma:
# options(OutDec = ",")
print(xtable(comp, caption = "Income according to sex", label = "tab:incsx"))
}

\name{plot.CrossTable}
\alias{plot.CrossTable}
\title{Mosaic plot from object of class CrossTable}
\description{
  This function receives a \code{\link{CrossTable}} object as its main
  argument and produces a mosaicplot.
}
\usage{
\method{plot}{CrossTable}(x, xlab, ylab, main = "", col,
           inv.x = FALSE, inv.y = FALSE, \dots)
}

\arguments{
  \item{x}{A object of class CrossTable.}
  \item{xlab}{See \code{\link[graphics]{plot.default}}.}
  \item{ylab}{See \code{\link[graphics]{plot.default}}.}
  \item{main}{See \code{\link[graphics]{plot.default}} and \code{\link[graphics]{title}}.}
  \item{col}{A specification for the default plotting color. (See section
    \sQuote{Color Specification} of \code{\link[graphics]{par}}). If the
    argument is missing, a gray scale is used to make the plot easier to
    interpret.}
  \item{inv.x}{A logical value indicating whether the order of the levels of
    the \code{x} variable should be inverted.}
  \item{inv.y}{A logical value indicating whether the order of the levels of
    the \code{y} variable should be inverted.}
  \item{\dots}{Further arguments to be passed to
    \code{\link[graphics]{mosaicplot}}.}
}

\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\seealso{\code{\link{CrossTable}}, \code{\link{crosstab}}.}

\name{data.frame2txt}
\alias{data.frame2txt}
\title{Export a data.frame and create scripts to input the data again.}
\description{
  Export a data.frame to a tab delimited text and create R and SPSS/PSPP
  scripts to input the data again.
}
\usage{
data.frame2txt(x, datafile = "x.txt", r.codefile = "x.R",
               sps.codefile = "x.sps", df.name = "x",
               user.missing)
}
\arguments{
  \item{x}{The data.frame to be exported.}
  \item{datafile}{The name of the tab delimited file to be created.}
  \item{r.codefile}{The name of the R script to read the data file.}
  \item{sps.codefile}{The name of the SPSS/PSPP script to read the data file.}
  \item{df.name}{The name of the data.frame object to be created by the R
    script.}
  \item{user.missing}{Labels of levels that must be coded as user missing
    in the sps script.}
}

\value{
  The return value of \code{\link[utils]{write.table}}.
}

\details{Logical vectors are converted into numeric before being saved.}

\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\examples{
\dontrun{
data(CO2)
data.frame2txt(CO2)
}
}

\encoding{UTF-8}
\name{LogRegR2}
\alias{LogRegR2}
\title{Pseudo R2 of logistic regression}
\description{
  The function calculates multiple R2 analogues (pseudo R2) of logistic
  regression.
}
\usage{
LogRegR2(model)
}
\arguments{
  \item{model}{A logistic regression model.}
}

\details{The function calculates McFaddens R2, Cox & Snell Index, and
  Nagelkerke Index of a logistic regression model.
}

\value{
  A object of class \code{list} with the calculated indexes.
}

\author{Dirk Enzmann}

\name{labels2R}
\alias{labels2R}
\title{Conversion of specially written text file into R code}
\description{
  Convert a specially written text file with information on variable labels
  and value labels into R code that converts integer vectors into factor
  variables.
}
\usage{
labels2R(lfile, rfile, dfname = "b", echo = FALSE)
}
\arguments{
  \item{lfile}{The path to the text file to be converted.}
  \item{rfile}{The path to the file to be created.}
  \item{dfname}{Name of data.frame where the variables are.}
  \item{echo}{If \code{TRUE}, then lines of lfile are printed in the R Console
    while the file is parsed. This may be useful debugging.}
}

\value{
  NULL.
}

\details{
  The return value is NULL, but \code{rfile} is created if the function is
  successful. The file is an R code that converts numeric vectors into
  factors. The text file must have a format as in the example below:

  \preformatted{
  v1 Sex
  1 Female
  2 Male

  v2 Household income

  v3 Taking all things together, would you say you are...
  1 Very happy
  2 Rather happy
  3 Not very happy
  4 Not at all happy
  }

  The above code would be converted into:

  \preformatted{
  b$v1 <- factor(b$v1, levels=c(1, 2), labels=c("Female", "Male"))
  attr(b$v1, "label") <- "Sex"
  attr(b$v2, "label") <- "Household income"
  b$v3 <- factor(b$v3, levels=c(1, 2, 3, 4),
                 labels=c("Very happy", "Rather happy",
                          "Not very happy", "Not at all happy"))
  attr(b$v3, "label") <- "Taking all things together, would you say you are..."
  }

}

\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}

\name{histkdnc}
\alias{histkdnc}
\title{Histogram with kernel density and normal curve}
\description{
  Plots a histogram with kernel density and normal curve.
}
\usage{
histkdnc(v, breaks = 0, include.lowest = TRUE, right = TRUE,
         main = "Histogram with kernel density and normal curve",
         xlab = deparse(substitute(v)), col = grey(0.90),
         col.cur = c("red", "blue"), lty.cur = c(1, 1),
         xlim = NULL, ylim = NULL, ...) 
}
\arguments{
  \item{v}{The object from which the histogram is desired.}
  \item{breaks}{See \link[graphics]{hist}.}
  \item{include.lowest}{See \link[graphics]{hist}.}
  \item{right}{See \link[graphics]{hist}.}
  \item{main}{See \link[graphics]{hist}.}
  \item{xlab}{See \link[graphics]{hist}.}
  \item{col}{See \link[graphics]{hist}.}
  \item{col.cur}{Vector of size two with the colors of, respectively, kernel
    density and normal curve.}
  \item{lty.cur}{Vector of size two with line type of, respectively, kernel
    density and normal curve.}
  \item{xlim}{See \link[graphics]{plot.default} and \link[graphics]{hist}.}
  \item{ylim}{See \link[graphics]{plot.default} and \link[graphics]{hist}.}
  \item{\dots}{Further arguments to be passed to \link[graphics]{hist}.}
}


\details{
  The function plots a histogram of the object \code{x} with its kernel density
  and a normal curve with the same mean and standard deviation of \code{x}.
}
\value{
  NULL.
}
\author{Dirk Enzmann (modified by Jakson Aquino\email{jalvesaq@gmail.com}).}

\name{fromUTF8}
\alias{fromUTF8}

\title{Conversion from UTF-8 encoding}
\description{Converts the encoding of some attributes of an object from UTF-8 into other
  encoding.}
\usage{
fromUTF8(x, to = "WINDOWS-1252") 
}
\arguments{
  \item{x}{A R object, usually a variable of a data frame or a data frame.}
  \item{to}{A string indicating the desired encoding. Common values are
    \code{"LATIN1"} and \code{"WINDOWS-1252"}. Type \code{iconvlist()} for the
    complete list of available encodings.}
}
\details{
  The function converts the attribute \code{label} of \code{x} from UTF-8 into the
  specified encoding. If \code{x} is a factor, the levels are converted as well.
  If \code{x} is a data.frame, the function makes the conversions in all of its
  variables.
}
\value{
  The object with its label and levels converted.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}.}

\seealso{\link[base]{iconv}}

\name{file.head}
\alias{file.head}
\title{Prints first lines of a file.}
\description{
  The function prints the first lines of a file, optionally truncating the lines
  according to the screen width. The lines are truncated at
  \code{getOption("width") - 2}.
}
\usage{
file.head(file, n, truncate.cols = TRUE)
}
\arguments{
  \item{file}{Character: The name of the file whose first lines should be
    printed.}
  \item{n}{The number of lines to show.}
  \item{truncate.cols}{Logical: if \code{TRUE} truncate the lines.}
}
\value{
  \code{NULL}.
}
\author{Jakson A. Aquino \email{jalvesaq@gmail.com}}
